import unittest
from pyvalidator import *

class TestIsNumber(unittest.TestCase):

    def test_valid_numbers(self):
        self.assertTrue(is_number('13'))
        self.assertTrue(is_number('123'))
        self.assertTrue(is_number('0.11'))
        self.assertTrue(is_number('-0'))
        self.assertTrue(is_number('+1.1'))
        self.assertTrue(is_number('012'))
        self.assertTrue(is_number('-013'))
        self.assertTrue(is_number('000'))
        self.assertTrue(is_number('123.123'))
        self.assertTrue(is_number('-123٫123'))
        self.assertTrue(is_number('+0٫123'))
        self.assertTrue(is_number('-0.11250738585072011e-307'))
        print('OK - test_valid_numbers')

    def test_invalid_numbers(self):
        self.assertFalse(is_number('123,123,asd'))
        self.assertFalse(is_number('asd'))
        self.assertFalse(is_number('.'))
        self.assertFalse(is_number('   '))
        self.assertFalse(is_number(''))
        self.assertFalse(is_number(','))
        self.assertFalse(is_number('foofoo'))
        self.assertFalse(is_number('2^3'))
        self.assertFalse(is_number('2+3'))
        print('OK - test_invalid_numbers')



import urllib.parse as parse

from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.merge import merge

default_options = {
    "min": 0,
    "max": None
}

def is_byte_length(input: str, options = {}) -> bool:
    assert_string(input)

    options = merge(options, default_options)

    pattern = RegEx("%..|.")

    split_input = pattern.split(parse.quote(input, safe='@/:;?&=+$,'))

    input_length = len(split_input) - 1

    return input_length >= options["min"] and (
        options["max"] == None or input_length <= options["max"]
    )



import unittest
from pyvalidator import *

class TestIsMongoId(unittest.TestCase):

    def test_valid_mongo_id(self):
        self.assertTrue(is_mongo_id('507f1f77bcf86cd799439011'))
        self.assertTrue(is_mongo_id('5349b4ddd2781d08c09890f3'))
        self.assertTrue(is_mongo_id('5349b4ddd2781d08c09890f4'))
        print('OK - test_valid_mongo_id')

    def test_invalid_mongo_id(self):
        self.assertFalse(is_mongo_id(''))
        self.assertFalse(is_mongo_id('.'))
        self.assertFalse(is_mongo_id('                        '))
        self.assertFalse(is_mongo_id('507f1f77bcf86cd7994390'))
        self.assertFalse(is_mongo_id('507f1f77bcf86cd79943901z'))
        self.assertFalse(is_mongo_id(' 507f1f77bcf86cd799439011'))
        self.assertFalse(is_mongo_id('507f1f77bcf86cd799439011 '))
        print('OK - test_invalid_mongo_id')



import unittest
from pyvalidator import *

class TestIsRgbColor(unittest.TestCase):

    def test_valid_rgb_color(self):
        self.assertTrue(is_rgb_color('rgb(0,0,0)'))
        self.assertTrue(is_rgb_color('rgb(255,255,255)'))
        self.assertTrue(is_rgb_color('rgba(0,0,0,0)'))
        self.assertTrue(is_rgb_color('rgba(255,255,255,1)'))
        self.assertTrue(is_rgb_color('rgba(255,255,255,.1)'))
        self.assertTrue(is_rgb_color('rgba(255,255,255,0.1)'))
        self.assertTrue(is_rgb_color('rgb(5%,5%,5%)'))
        self.assertTrue(is_rgb_color('rgba(5%,5%,5%,.3)'))
        print('OK - test_valid_rgb_color')

    def test_invalid_rgb_color(self):
        self.assertFalse(is_rgb_color('rgb(0,0,0,)'))
        self.assertFalse(is_rgb_color('rgb(0,0,)'))
        self.assertFalse(is_rgb_color('rgb(0,0,256)'))
        self.assertFalse(is_rgb_color('rgb()'))
        self.assertFalse(is_rgb_color('rgba(0,0,0)'))
        self.assertFalse(is_rgb_color('rgba(255,255,255,2)'))
        self.assertFalse(is_rgb_color('rgba(255,255,255,.12)'))
        self.assertFalse(is_rgb_color('rgba(255,255,256,0.1)'))
        self.assertFalse(is_rgb_color('rgb(4,4,5%)'))
        self.assertFalse(is_rgb_color('rgba(5%,5%,5%)'))
        self.assertFalse(is_rgb_color('rgba(3,3,3%,.3)'))
        self.assertFalse(is_rgb_color('rgb(101%,101%,101%)'))
        self.assertFalse(is_rgb_color('rgba(3%,3%,101%,0.3)'))
        print('OK - test_invalid_rgb_color')

    def test_valid_rgb_color_no_percent(self):
        self.assertTrue(is_rgb_color('rgb(1,1,1)', False))
        self.assertTrue(is_rgb_color('rgba(5,5,5,.3)', False))
        print('OK - test_valid_rgb_color_no_percent')

    def test_invalid_rgb_color_no_percent(self):
        self.assertFalse(is_rgb_color('rgb(4,4,5%)', False))
        self.assertFalse(is_rgb_color('rgba(1%,1%,1%)', False))
        print('OK - test_invalid_rgb_color_no_percent')



import unittest
from pyvalidator import *

class TestIsCurrency(unittest.TestCase):

    def test_valid_currency(self):
        self.assertTrue(is_currency('-$10,123.45'))
        self.assertTrue(is_currency('$10,123.45'))
        self.assertTrue(is_currency('$10123.45'))
        self.assertTrue(is_currency('10,123.45'))
        self.assertTrue(is_currency('10123.45'))
        self.assertTrue(is_currency('10,123'))
        self.assertTrue(is_currency('1,123,456'))
        self.assertTrue(is_currency('1123456'))
        self.assertTrue(is_currency('1.39'))
        self.assertTrue(is_currency('.03'))
        self.assertTrue(is_currency('0.10'))
        self.assertTrue(is_currency('$0.10'))
        self.assertTrue(is_currency('-$0.01'))
        self.assertTrue(is_currency('-$.99'))
        self.assertTrue(is_currency('$100,234,567.89'))
        self.assertTrue(is_currency('$10,123'))
        self.assertTrue(is_currency('10,123'))
        self.assertTrue(is_currency('-10123'))
        print('OK - test_valid_currency')

    def test_invalid_currency(self):
        self.assertFalse(is_currency('1.234'))
        self.assertFalse(is_currency('$1.1'))
        self.assertFalse(is_currency('$ 32.50'))
        self.assertFalse(is_currency('500$'))
        self.assertFalse(is_currency('.0001'))
        self.assertFalse(is_currency('$.001'))
        self.assertFalse(is_currency('$0.001'))
        self.assertFalse(is_currency('$12,34.56'))
        self.assertFalse(is_currency('123456,123,123456'))
        self.assertFalse(is_currency('123,4'))
        self.assertFalse(is_currency(',123'))
        self.assertFalse(is_currency('$-,123'))
        self.assertFalse(is_currency('$'))
        self.assertFalse(is_currency('.'))
        self.assertFalse(is_currency(','))
        self.assertFalse(is_currency('00'))
        self.assertFalse(is_currency('$-'))
        self.assertFalse(is_currency('$-,.'))
        self.assertFalse(is_currency('-'))
        self.assertFalse(is_currency('-$'))
        self.assertFalse(is_currency(''))
        self.assertFalse(is_currency('- $'))
        print('OK - test_invalid_currency')

    def test_valid_currency_dont_allow_decimal(self):
        self.assertTrue(is_currency('-$10,123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('$10,123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('$10123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('10,123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('10123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('10,123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('1,123,456', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('1123456', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('1', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('0', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('$0', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('-$0', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('$100,234,567', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('$10,123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('10,123', {
          "allow_decimal": False,
        }))
        self.assertTrue(is_currency('-10123', {
          "allow_decimal": False,
        }))
        print('OK - test_valid_currency_dont_allow_decimal')

    def test_invalid_currency_allow_decimal(self):
        self.assertFalse(is_currency('1.234', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$1.1', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$ 32.50', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('500$', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('.0001', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$.001', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$0.001', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$12,34.56', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('123456,123,123456', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('123,4', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency(',123', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$-,123', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('.', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency(',', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('00', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$-', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('$-,.', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('-', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('-$', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('', {
          "allow_decimal": False,
        }))
        self.assertFalse(is_currency('- $', {
          "allow_decimal": False,
        }))
        print('OK - test_invalid_currency_dont_allow_decimal')

    def test_valid_currency_digits_after_decimal(self):
        self.assertTrue(is_currency('-$10,123.4', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertTrue(is_currency('$10,123.454', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertTrue(is_currency('$10123.452', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertTrue(is_currency('10,123.453', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertTrue(is_currency('10123.450', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertTrue(is_currency('10,123', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertTrue(is_currency('1,123,456', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertTrue(is_currency('0.100', {
          "digits_after_decimal": [1, 3],
        }))
        print('OK - test_valid_currency_dont_allow_decimal')

    def test_invalid_currency_digits_after_decimal(self):
        self.assertFalse(is_currency('1.23', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('$1.13322', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('$ 32.50', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('500$', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('.0001', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('$.01', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('$0.01', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('$12,34.56', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('123456,123,123456', {
          "digits_after_decimal": [1, 3],
        }))
        self.assertFalse(is_currency('123,4', {
          "digits_after_decimal": [1, 3],
        }))
        print('OK - test_invalid_currency_digits_after_decimal')

    def test_valid_currency_require_symbol(self):
        self.assertTrue(is_currency('-$.99', {
          "require_symbol": True,
        }))
        self.assertTrue(is_currency('$100,234,567.89', {
          "require_symbol": True,
        }))
        self.assertTrue(is_currency('$1123456', {
          "require_symbol": True,
        }))
        print('OK - test_valid_currency_require_symbol')

    def test_invalid_currency_require_symbol(self):
        self.assertFalse(is_currency('1.23', {
          "require_symbol": True,
        }))
        self.assertFalse(is_currency('1.1', {
          "require_symbol": True,
        }))
        self.assertFalse(is_currency('500', {
          "require_symbol": True,
        }))
        self.assertFalse(is_currency('.0001', {
          "require_symbol": True,
        }))
        print('OK - test_invalid_currency_require_symbol')

    def test_valid_currency_add_symbol_prefix(self):
        self.assertTrue(is_currency('123,456.78', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        self.assertTrue(is_currency('¥6,954,231', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        self.assertTrue(is_currency('¥-10.03', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        self.assertTrue(is_currency('.03', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        print('OK - test_valid_currency_add_symbol_prefix')

    def test_invalid_currency_add_symbol_prefix(self):
        self.assertFalse(is_currency('1.234', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        self.assertFalse(is_currency('¥1.1', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        self.assertFalse(is_currency('5,00', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        self.assertFalse(is_currency('.0001', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        self.assertFalse(is_currency('¥.001', {
          "symbol": "¥",
          "negative_sign_before_digits": True,
        }))
        print('OK - test_invalid_currency_add_symbol_prefix')



import unittest
from pyvalidator import *

class TestIsUppercase(unittest.TestCase):

    def test_valid_uppercase(self):
        self.assertTrue(is_uppercase('CCC'))
        self.assertTrue(is_uppercase('UYTREWQ'))
        self.assertTrue(is_uppercase('ASD123'))
        self.assertTrue(is_uppercase('HELLO WORLD'))
        self.assertTrue(is_uppercase('   .'))
        print('OK - test_valid_uppercase')

    def test_invalid_uppercase(self):
        self.assertFalse(is_uppercase('hello world'))
        self.assertFalse(is_uppercase('HELLO WORLd'))
        self.assertFalse(is_uppercase('asd123'))
        print('OK - test_invalid_uppercase')



import unittest
from pyvalidator import *

class TestIsDivisibleBy(unittest.TestCase):

    def test_valid_divisible(self):
        self.assertTrue(is_divisible_by('2', 2))
        self.assertTrue(is_divisible_by('4', 2))
        self.assertTrue(is_divisible_by('100', 2))
        self.assertTrue(is_divisible_by('1000', 2))
        print('OK - test_valid_divisible')

    def test_invalid_divisible(self):
        self.assertFalse(is_divisible_by('1', 2))
        self.assertFalse(is_divisible_by('2.5', 2))
        self.assertFalse(is_divisible_by('101', 2))
        self.assertFalse(is_divisible_by('foo', 2))
        self.assertFalse(is_divisible_by('', 2))
        self.assertFalse(is_divisible_by('2020-01-06T14:31:00.135Z', 2))
        print('OK - test_invalid_divisible')



import unittest
from pyvalidator import *

class TestIsCreditCard(unittest.TestCase):

    def test_valid_credit_cards(self):
        self.assertTrue(is_credit_card('375556917985515'))
        self.assertTrue(is_credit_card('36050234196908'))
        self.assertTrue(is_credit_card('4716461583322103'))
        self.assertTrue(is_credit_card('4716-2210-5188-5662'))
        self.assertTrue(is_credit_card('4929 7226 5379 7141'))
        self.assertTrue(is_credit_card('5398228707871527'))
        self.assertTrue(is_credit_card('6283875070985593'))
        self.assertTrue(is_credit_card('6263892624162870'))
        self.assertTrue(is_credit_card('6234917882863855'))
        self.assertTrue(is_credit_card('6234698580215388'))
        self.assertTrue(is_credit_card('6226050967750613'))
        self.assertTrue(is_credit_card('6246281879460688'))
        self.assertTrue(is_credit_card('2222155765072228'))
        self.assertTrue(is_credit_card('2225855203075256'))
        self.assertTrue(is_credit_card('2720428011723762'))
        self.assertTrue(is_credit_card('2718760626256570'))
        self.assertTrue(is_credit_card('6765780016990268'))
        self.assertTrue(is_credit_card('4716989580001715211'))
        self.assertTrue(is_credit_card('8171999927660000'))
        self.assertTrue(is_credit_card('8171999900000000021'))
        print('OK - test_valid_credit_cards')

    def test_invalid_credit_cards(self):
        self.assertFalse(is_credit_card('foo'))
        self.assertFalse(is_credit_card('5398228707871528'))
        self.assertFalse(is_credit_card('2718760626256571'))
        self.assertFalse(is_credit_card('2721465526338453'))
        self.assertFalse(is_credit_card('2220175103860763'))
        self.assertFalse(is_credit_card('375556917985515999999993'))
        self.assertFalse(is_credit_card('899999996234917882863855'))
        self.assertFalse(is_credit_card('prefix6234917882863855'))
        self.assertFalse(is_credit_card('6234917882863855suffix'))
        self.assertFalse(is_credit_card('4716989580001715213'))
        print('OK - test_invalid_credit_cards')



import unittest
from pyvalidator import *

class TestIsImei(unittest.TestCase):

    def test_valid_imei(self):
        self.assertTrue(is_imei('352099001761481'))
        self.assertTrue(is_imei('868932036356090'))
        self.assertTrue(is_imei('490154203237518'))
        self.assertTrue(is_imei('546918475942169'))
        self.assertTrue(is_imei('998227667144730'))
        self.assertTrue(is_imei('532729766805999'))
        print('OK - test_valid_imei')

    def test_invalid_imei(self):
        self.assertFalse(is_imei('490154203237517'))
        self.assertFalse(is_imei('3568680000414120'))
        self.assertFalse(is_imei('3520990017614823'))
        print('OK - test_invalid_imei')

    def test_valid_imei_with_hyphens(self):
        self.assertTrue(is_imei('35-209900-176148-1', { "allow_hyphens": True }))
        self.assertTrue(is_imei('86-893203-635609-0', { "allow_hyphens": True }))
        self.assertTrue(is_imei('49-015420-323751-8', { "allow_hyphens": True }))
        self.assertTrue(is_imei('54-691847-594216-9', { "allow_hyphens": True }))
        self.assertTrue(is_imei('99-822766-714473-0', { "allow_hyphens": True }))
        self.assertTrue(is_imei('53-272976-680599-9', { "allow_hyphens": True }))
        print('OK - test_valid_imei_with_hyphens')

    def test_invalid_imei_with_hyphens(self):
        self.assertFalse(is_imei('49-015420-323751-7', { "allow_hyphens": True }))
        self.assertFalse(is_imei('35-686800-0041412-0', { "allow_hyphens": True }))
        self.assertFalse(is_imei('35-209900-1761482-3', { "allow_hyphens": True }))
        self.assertFalse(is_imei('3520990017614823', { "allow_hyphens": True }))
        print('OK - test_invalid_imei_with_hyphens')



import unittest
from pyvalidator import *

class TestIsEthereumAddress(unittest.TestCase):

    def test_valid_ethereum_address(self):
        self.assertTrue(is_ethereum_address('0x0000000000000000000000000000000000000001'))
        self.assertTrue(is_ethereum_address('0x683E07492fBDfDA84457C16546ac3f433BFaa128'))
        self.assertTrue(is_ethereum_address('0x88dA6B6a8D3590e88E0FcadD5CEC56A7C9478319'))
        self.assertTrue(is_ethereum_address('0x8a718a84ee7B1621E63E680371e0C03C417cCaF6'))
        self.assertTrue(is_ethereum_address('0xFCb5AFB808b5679b4911230Aa41FfCD0cd335b42'))
        print('OK - test_valid_ethereum_address')

    def test_invalid_ethereum_address(self):
        self.assertFalse(is_ethereum_address('0xGHIJK05pwm37asdf5555QWERZCXV2345AoEuIdHt'))
        self.assertFalse(is_ethereum_address('0xFCb5AFB808b5679b4911230Aa41FfCD0cd335b422222'))
        self.assertFalse(is_ethereum_address('0xFCb5AFB808b5679b4911230Aa41FfCD0cd33'))
        self.assertFalse(is_ethereum_address('0b0110100001100101011011000110110001101111'))
        self.assertFalse(is_ethereum_address('683E07492fBDfDA84457C16546ac3f433BFaa128'))
        self.assertFalse(is_ethereum_address('1C6o5CDkLxjsVpnLSuqRs1UBFozXLEwYvU'))
        print('OK - test_invalid_ethereum_address')



import unittest
from pyvalidator import *

class TestIsFullWidth(unittest.TestCase):

    def test_valid_full_width(self):
        self.assertTrue(is_full_width('ひらがな・カタカナ、．漢字'))
        self.assertTrue(is_full_width('３ー０　ａ＠ｃｏｍ'))
        self.assertTrue(is_full_width('Ｆｶﾀｶﾅﾞﾬ'))
        self.assertTrue(is_full_width('Ğood＝Parts'))
        print('OK - test_valid_full_width')

    def test_invalid_full_width(self):
        self.assertFalse(is_full_width('abc'))
        self.assertFalse(is_full_width('abc123'))
        self.assertFalse(is_full_width('!"#$%&()<>/+=-_? ~^|.,@`{}[]'))
        print('OK - test_invalid_full_width')



from .utils.assert_string import assert_string
from .utils.Classes.RegEx import RegEx

isrc_pattern = RegEx("^[A-Z]{2}[0-9A-Z]{3}\d{2}\d{5}$")

def is_isrc(input: str) -> bool:
    input = assert_string(input)
    return isrc_pattern.match(input)



import unittest
from pyvalidator import *

class TestIsLatLong(unittest.TestCase):

    def test_valid_lat_long(self):
        self.assertTrue(is_lat_long('(-17.738223, 85.605469)'))
        self.assertTrue(is_lat_long('(-12.3456789, +12.3456789)'))
        self.assertTrue(is_lat_long('(-60.978437, -0.175781)'))
        self.assertTrue(is_lat_long('(77.719772, -37.529297)'))
        self.assertTrue(is_lat_long('(7.264394, 165.058594)'))
        self.assertTrue(is_lat_long('0.955766, -19.863281'))
        self.assertTrue(is_lat_long('(31.269161,164.355469)'))
        self.assertTrue(is_lat_long('+12.3456789, -12.3456789'))
        self.assertTrue(is_lat_long('-15.379543, -137.285156'))
        self.assertTrue(is_lat_long('(11.770570, -162.949219)'))
        self.assertTrue(is_lat_long('-55.034319, 113.027344'))
        self.assertTrue(is_lat_long('58.025555, 36.738281'))
        self.assertTrue(is_lat_long('55.720923,-28.652344'))
        self.assertTrue(is_lat_long('-90.00000,-180.00000'))
        self.assertTrue(is_lat_long('(-71, -146)'))
        self.assertTrue(is_lat_long('(-71.616864, -146.616864)'))
        self.assertTrue(is_lat_long('-0.55, +0.22'))
        self.assertTrue(is_lat_long('90, 180'))
        self.assertTrue(is_lat_long('+90, -180'))
        self.assertTrue(is_lat_long('-90,+180'))
        self.assertTrue(is_lat_long('90,180'))
        self.assertTrue(is_lat_long('0, 0'))
        print('OK - test_valid_lat_long')

    def test_invalid_lat_long(self):
        self.assertFalse(is_lat_long('(020.000000, 010.000000000)'))
        self.assertFalse(is_lat_long('89.9999999989, 360.0000000'))
        self.assertFalse(is_lat_long('90.1000000, 180.000000'))
        self.assertFalse(is_lat_long('+90.000000, -180.00001'))
        self.assertFalse(is_lat_long('090.0000, 0180.0000'))
        self.assertFalse(is_lat_long('126, -158'))
        self.assertFalse(is_lat_long('(-126.400010, -158.400010)'))
        self.assertFalse(is_lat_long('-95, -96'))
        self.assertFalse(is_lat_long('-95.738043, -96.738043'))
        self.assertFalse(is_lat_long('137, -148'))
        self.assertFalse(is_lat_long('(-137.5942, -148.5942)'))
        self.assertFalse(is_lat_long('(-120, -203)'))
        self.assertFalse(is_lat_long('(-119, -196)'))
        self.assertFalse(is_lat_long('+119.821728, -196.821728'))
        self.assertFalse(is_lat_long('(-110, -223)'))
        self.assertFalse(is_lat_long('-110.369532, 223.369532'))
        self.assertFalse(is_lat_long('(-120.969949, +203.969949)'))
        self.assertFalse(is_lat_long('-116, -126'))
        self.assertFalse(is_lat_long('-116.894222, -126.894222'))
        self.assertFalse(is_lat_long('-112, -160'))
        self.assertFalse(is_lat_long('-112.96381, -160.96381'))
        self.assertFalse(is_lat_long('-90., -180.'))
        self.assertFalse(is_lat_long('+90.1, -180.1'))
        self.assertFalse(is_lat_long('(-17.738223, 85.605469'))
        self.assertFalse(is_lat_long('0.955766, -19.863281)'))
        self.assertFalse(is_lat_long('+,-'))
        self.assertFalse(is_lat_long('(,)'))
        self.assertFalse(is_lat_long(','))
        self.assertFalse(is_lat_long(' '))
        print('OK - test_invalid_lat_long')

    def test_valid_lat_long_dms(self):
        self.assertTrue(is_lat_long('40° 26′ 46″ N, 79° 58′ 56″ W', { "check_dms": True }))
        self.assertTrue(is_lat_long('40° 26′ 46″ S, 79° 58′ 56″ E', { "check_dms": True }))
        self.assertTrue(is_lat_long('90° 0′ 0″ S, 180° 0′ 0″ E', { "check_dms": True }))
        self.assertTrue(is_lat_long('40° 26′ 45.9996″ N, 79° 58′ 55.2″ E', { "check_dms": True }))
        self.assertTrue(is_lat_long('40° 26′ 46″ n, 79° 58′ 56″ w', { "check_dms": True }))
        self.assertTrue(is_lat_long('40°26′46″s, 79°58′56″e', { "check_dms": True }))
        self.assertTrue(is_lat_long('11° 0′ 0.005″ S, 180° 0′ 0″ E', { "check_dms": True }))
        self.assertTrue(is_lat_long('40°26′45.9996″N, 79°58′55.2″E', { "check_dms": True }))
        print('OK - test_valid_lat_long_dms')

    def test_invalid_lat_long_dms(self):
        self.assertFalse(is_lat_long('100° 26′ 46″ N, 79° 70′ 56″ W', { "check_dms": True }))
        self.assertFalse(is_lat_long('40° 89′ 46″ S, 79° 58′ 100″ E', { "check_dms": True }))
        self.assertFalse(is_lat_long('40° 26.445′ 45″ N, 79° 58′ 55.2″ E', { "check_dms": True }))
        self.assertFalse(is_lat_long('40° 46″ N, 79° 58′ 56″ W', { "check_dms": True }))
        print('OK - test_invalid_lat_long_dms')



import unittest
from pyvalidator import *

class TestIsEan(unittest.TestCase):

    def test_valid_ean(self):
        self.assertTrue(is_ean('9421023610112'))
        self.assertTrue(is_ean('1234567890128'))
        self.assertTrue(is_ean('4012345678901'))
        self.assertTrue(is_ean('9771234567003'))
        self.assertTrue(is_ean('9783161484100'))
        self.assertTrue(is_ean('73513537'))
        self.assertTrue(is_ean('00012345600012'))
        self.assertTrue(is_ean('10012345678902'))
        self.assertTrue(is_ean('20012345678909'))
        print('OK - test_valid_ean')

    def test_invalid_ean(self):
        self.assertFalse(is_ean('5901234123451'))
        self.assertFalse(is_ean('079777681629'))
        self.assertFalse(is_ean('0705632085948'))
        print('OK - test_invalid_ean')



import unittest
from pyvalidator import *

class TestIsEmoji(unittest.TestCase):

    def test_valid_emoji(self):
        self.assertTrue(is_emoji('🌼'))
        self.assertTrue(is_emoji('😀'))
        self.assertTrue(is_emoji('😃'))
        self.assertTrue(is_emoji('😎'))
        self.assertTrue(is_emoji('🍗'))
        self.assertTrue(is_emoji('🌯'))
        self.assertTrue(is_emoji('🍔'))
        self.assertTrue(is_emoji('🍗🍗🍗'))
        self.assertTrue(is_emoji('💪'))
        self.assertTrue(is_emoji('🇷🇸'))
        self.assertTrue(is_emoji('🇺🇸'))
        self.assertTrue(is_emoji('🇦🇪'))
        self.assertTrue(is_emoji('🇮🇱'))
        self.assertTrue(is_emoji('🇮🇹'))
        self.assertTrue(is_emoji('🇹🇷'))
        self.assertTrue(is_emoji('🇦🇷'))
        self.assertTrue(is_emoji('🇧🇧'))
        print('OK - test_valid_emoji')

    def test_invalid_emoji(self):
        self.assertFalse(is_emoji(''))
        self.assertFalse(is_emoji(' '))
        self.assertFalse(is_emoji('\n'))
        self.assertFalse(is_emoji('-1💪'))
        self.assertFalse(is_emoji('💪65536'))
        self.assertFalse(is_emoji('655369'))
        self.assertFalse(is_emoji('.'))
        self.assertFalse(is_emoji('😃.😃'))
        self.assertFalse(is_emoji('asd'))
        self.assertFalse(is_emoji('\U000027B00'))
        print('OK - test_invalid_emoji')

    def test_valid_emoji_by_omited_dot(self):
        self.assertTrue(is_emoji('.🌼', { "omit_rule": r'[.]' }))
        self.assertTrue(is_emoji('😀.', { "omit_rule": r'[.]' }))
        self.assertTrue(is_emoji('😀.🌼', { "omit_rule": r'[.]' }))
        self.assertTrue(is_emoji('😀.🌼.😀', { "omit_rule": r'[.]' }))
        self.assertTrue(is_emoji('😀.-🌼.-😀', { "omit_rule": r'[.-]' }))
        print('OK - test_valid_emoji_by_omited_dot')



from typing import Literal

from .utils.assert_string import assert_string
from .utils.Classes.RegEx import RegEx
from .utils.includes import includesNot

ip_v4_segment= "(?:[0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])"
ip_v4_address = "({}[.]){{3}}{}".format(ip_v4_segment, ip_v4_segment)
ip_v4_address_pattern = RegEx('^' + ip_v4_address + '$')

ip_v6_segment_format = "(?:[0-9a-fA-F]{1,4})"
ip_v6_address_part_1 = "^((?:{}:){{7}}(?:{}|:)|".format(ip_v6_segment_format, ip_v6_segment_format)
ip_v6_address_part_2 = "(?:{}:){{6}}(?:{}|:{}|:)|".format(ip_v6_segment_format, ip_v4_address, ip_v6_segment_format)
ip_v6_address_part_3 = "(?:{}:){{5}}(?::{}|(:{}){{1,2}}|:)|".format(ip_v6_segment_format, ip_v4_address, ip_v6_segment_format)
ip_v6_address_part_4 = "(?:{}:){{4}}(?:(:{}){{0,1}}:{}|(:{}){{1,3}}|:)|".format(ip_v6_segment_format, ip_v6_segment_format, ip_v4_address, ip_v6_segment_format)
ip_v6_address_part_5 = "(?:{}:){{3}}(?:(:{}){{0,2}}:{}|(:{}){{1,4}}|:)|".format(ip_v6_segment_format, ip_v6_segment_format, ip_v4_address, ip_v6_segment_format)
ip_v6_address_part_6 = "(?:{}:){{2}}(?:(:{}){{0,3}}:{}|(:{}){{1,5}}|:)|".format(ip_v6_segment_format, ip_v6_segment_format, ip_v4_address, ip_v6_segment_format)
ip_v6_address_part_7 = "(?:{}:){{1}}(?:(:{}){{0,4}}:{}|(:{}){{1,6}}|:)|".format(ip_v6_segment_format, ip_v6_segment_format, ip_v4_address, ip_v6_segment_format)
ip_v6_address_part_8 = "(?::((?::{}){{0,5}}:{}|(?::{}){{1,7}}|:))".format(ip_v6_segment_format, ip_v4_address, ip_v6_segment_format)
ip_v6_address_part_9 = ")(%[0-9a-zA-Z-.:]{1,})?$"
ip_v6_address = (
    ip_v6_address_part_1 +
    ip_v6_address_part_2 +
    ip_v6_address_part_3 +
    ip_v6_address_part_4 +
    ip_v6_address_part_5 +
    ip_v6_address_part_6 +
    ip_v6_address_part_7 +
    ip_v6_address_part_8 +
    ip_v6_address_part_9
)
ip_v6_address_pattern = RegEx(ip_v6_address)

ip_versions = [4, 6, '4', '6']

def is_ip(input: str, version: Literal[4, 6, '4', '6', None] = None) -> bool:
    input = assert_string(input)

    if version == None:
        return is_ip(input, 4) or is_ip(input, 6)

    if includesNot(ip_versions, version):
        return False

    if version == 6 or version == '6':
        return ip_v6_address_pattern.match(input)

    if not ip_v4_address_pattern.match(input):
        return False

    parts = input.split('.')
    parts.sort()

    try:
        return int(parts[3]) <= 255
    except:
        return False






import unittest
from pyvalidator import *

class TestIsSlug(unittest.TestCase):

    def test_valid_slug(self):
        self.assertTrue(is_slug('foo'))
        self.assertTrue(is_slug('foo-bar'))
        self.assertTrue(is_slug('foo_bar'))
        self.assertTrue(is_slug('foo-bar-foo'))
        self.assertTrue(is_slug('foo-bar_foo'))
        self.assertTrue(is_slug('foo-bar_foo*75-b4r-**_foo'))
        self.assertTrue(is_slug('foo-bar_foo*75-b4r-**_foo-&&'))
        print('OK - test_valid_slug')

    def test_invalid_slug(self):
        self.assertFalse(is_slug('not-----------slug'))
        self.assertFalse(is_slug('@#_$@'))
        self.assertFalse(is_slug('-not-slug'))
        self.assertFalse(is_slug('not-slug-'))
        self.assertFalse(is_slug('_not-slug'))
        self.assertFalse(is_slug('not-slug_'))
        self.assertFalse(is_slug('not slug'))
        print('OK - test_invalid_slug')



import unittest
from pyvalidator import *

class TestIsDecimal(unittest.TestCase):

    def test_valid_decimale(self):
        self.assertTrue(is_decimal('123'))
        self.assertTrue(is_decimal('00123'))
        self.assertTrue(is_decimal('-00123'))
        self.assertTrue(is_decimal('0'))
        self.assertTrue(is_decimal('-0'))
        self.assertTrue(is_decimal('+123'))
        self.assertTrue(is_decimal('0.01'))
        self.assertTrue(is_decimal('.1'))
        self.assertTrue(is_decimal('1.0'))
        self.assertTrue(is_decimal('-.25'))
        self.assertTrue(is_decimal('-0'))
        self.assertTrue(is_decimal('0.0000000000001'))
        print('OK - test_valid_decimale')

    def test_invalid_decimale(self):
        self.assertFalse(is_decimal('0,01'))
        self.assertFalse(is_decimal(',1'))
        self.assertFalse(is_decimal('1,0'))
        self.assertFalse(is_decimal('-,25'))
        self.assertFalse(is_decimal('0,0000000000001'))
        self.assertFalse(is_decimal('0٫01'))
        self.assertFalse(is_decimal('٫1'))
        self.assertFalse(is_decimal('1٫0'))
        self.assertFalse(is_decimal('-٫25'))
        self.assertFalse(is_decimal('0٫0000000000001'))
        self.assertFalse(is_decimal('....'))
        self.assertFalse(is_decimal(' '))
        self.assertFalse(is_decimal('-'))
        self.assertFalse(is_decimal('+'))
        self.assertFalse(is_decimal('.'))
        self.assertFalse(is_decimal('0.1a'))
        self.assertFalse(is_decimal('a'))
        self.assertFalse(is_decimal('\n'))
        print('OK - test_invalid_decimale')

    def test_valid_decimale_locale_en_au(self):
        self.assertTrue(is_decimal('123', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('00123', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('-00123', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('0', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('-0', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('+123', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('0.01', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('.1', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('1.0', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('-.25', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('-0', { "locale": "en-AU" }))
        self.assertTrue(is_decimal('0.0000000000001', { "locale": "en-AU" }))
        print('OK - test_valid_decimale_locale_en_au')

    def test_invalid_decimale_locale_en_au(self):
        self.assertFalse(is_decimal('0,01', { "locale": "en-AU" }))
        self.assertFalse(is_decimal(',1', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('1,0', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('-,25', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('0,0000000000001', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('0٫01', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('٫1', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('1٫0', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('-٫25', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('0٫0000000000001', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('....', { "locale": "en-AU" }))
        self.assertFalse(is_decimal(' ', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('-', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('+', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('.', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('0.1a', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('a', { "locale": "en-AU" }))
        self.assertFalse(is_decimal('\n', { "locale": "en-AU" }))
        print('OK - test_invalid_decimale_locale_en_au')

    def test_valid_decimale_locale_ar(self):
        self.assertTrue(is_decimal('123', { "locale": "ar" }))
        self.assertTrue(is_decimal('00123', { "locale": "ar" }))
        self.assertTrue(is_decimal('-00123', { "locale": "ar" }))
        self.assertTrue(is_decimal('0٫01', { "locale": "ar" }))
        self.assertTrue(is_decimal('٫1', { "locale": "ar" }))
        self.assertTrue(is_decimal('1٫0', { "locale": "ar" }))
        self.assertTrue(is_decimal('-٫25', { "locale": "ar" }))
        print('OK - test_valid_decimale_locale_ar')

    def test_invalid_decimale_locale_ar(self):
        self.assertFalse(is_decimal('0.01', { "locale": "ar" }))
        self.assertFalse(is_decimal('.1', { "locale": "ar" }))
        self.assertFalse(is_decimal('1.0', { "locale": "ar" }))
        self.assertFalse(is_decimal('-.25', { "locale": "ar" }))
        self.assertFalse(is_decimal('0.0000000000001', { "locale": "ar" }))
        self.assertFalse(is_decimal('0.01', { "locale": "ar" }))
        self.assertFalse(is_decimal('.1', { "locale": "ar" }))
        self.assertFalse(is_decimal('1.0', { "locale": "ar" }))
        self.assertFalse(is_decimal('-.25', { "locale": "ar" }))
        self.assertFalse(is_decimal('0.0000000000001', { "locale": "ar" }))
        print('OK - test_invalid_decimale_locale_ar')

    def test_valid_decimale_locale_force_decimal(self):
        self.assertTrue(is_decimal('0.01', { "force_decimal": True }))
        self.assertTrue(is_decimal('.1', { "force_decimal": True }))
        self.assertTrue(is_decimal('1.0', { "force_decimal": True }))
        self.assertTrue(is_decimal('-.25', { "force_decimal": True }))
        self.assertTrue(is_decimal('0.0000000000001', { "force_decimal": True }))
        print('OK - test_valid_decimale_locale_force_decimal')

    def test_invalid_decimale_locale_force_decimal(self):
        self.assertFalse(is_decimal('-0', { "force_decimal": True }))
        self.assertFalse(is_decimal('123', { "force_decimal": True }))
        self.assertFalse(is_decimal('00123', { "force_decimal": True }))
        self.assertFalse(is_decimal('0', { "force_decimal": True }))
        self.assertFalse(is_decimal('+123', { "force_decimal": True }))
        print('OK - test_invalid_decimale_locale_force_decimal')

    def test_valid_decimale_locale_decimal_digits_2_3(self):
        self.assertTrue(is_decimal('123', { "decimal_digits": "2,3" }))
        self.assertTrue(is_decimal('00123', { "decimal_digits": "2,3" }))
        self.assertTrue(is_decimal('-00123', { "decimal_digits": "2,3" }))
        self.assertTrue(is_decimal('0', { "decimal_digits": "2,3" }))
        self.assertTrue(is_decimal('-0', { "decimal_digits": "2,3" }))
        self.assertTrue(is_decimal('+123', { "decimal_digits": "2,3" }))
        self.assertTrue(is_decimal('-.255', { "decimal_digits": "2,3" }))
        print('OK - test_valid_decimale_locale_decimal_digits_2_3')

    def test_invalid_decimale_locale_decimal_digits_2_3(self):
        self.assertFalse(is_decimal('0.0000000000001', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('0.0', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('.1', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('1.0', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('-.2564', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('0.0', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('٫1', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('1٫0', { "decimal_digits": "2,3" }))
        self.assertFalse(is_decimal('-٫25', { "decimal_digits": "2,3" }))
        print('OK - test_valid_decimale_locale_decimal_digits_2_3')



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .is_number import is_number

credit_card = RegEx("^(?:4[0-9]{12}(?:[0-9]{3,6})?|5[1-5][0-9]{14}|(222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}|6(?:011|5[0-9][0-9])[0-9]{12,15}|3[47][0-9]{13}|3(?:0[0-5]|[68][0-9])[0-9]{11}|(?:2131|1800|35\d{3})\d{11}|6[27][0-9]{14}|^(81[0-9]{14,17}))$")

sanetization_pattern = RegEx("[- ]+", 'g')

def is_credit_card(input: str) -> bool:
    input = assert_string(input)

    sanitized = input.sub(sanetization_pattern, '')

    if not credit_card.match(sanitized):
        return False

    sum = 0
    should_double = False
    reversed_sanetized_input = sanitized[::-1]

    for digit in reversed_sanetized_input:
        if not is_number(digit):
            return False

        tmp_num = int(float(digit))

        if should_double:
            tmp_num = tmp_num * 2
            if tmp_num >= 10:
                sum += ((tmp_num % 10) + 1)
            else:
                sum += tmp_num
        else:
            sum += tmp_num
        should_double = not should_double

    return bool(sanitized if (sum % 10) == 0 else False)



import unittest
from pyvalidator import *

class TestIsHexadecimal(unittest.TestCase):

    def test_valid_hexadecimal(self):
        self.assertTrue(is_hexadecimal('AA'))
        self.assertTrue(is_hexadecimal('deadBEEF'))
        self.assertTrue(is_hexadecimal('ff0044'))
        self.assertTrue(is_hexadecimal('0xff0044'))
        self.assertTrue(is_hexadecimal('0XfF0044'))
        self.assertTrue(is_hexadecimal('0x0123456789abcDEF'))
        self.assertTrue(is_hexadecimal('0X0123456789abcDEF'))
        self.assertTrue(is_hexadecimal('0hfedCBA9876543210'))
        self.assertTrue(is_hexadecimal('0HfedCBA9876543210'))
        self.assertTrue(is_hexadecimal('0123456789abcDEF'))
        print('OK - test_valid_hexadecimal')

    def test_invalid_hexadecimal(self):
        self.assertFalse(is_hexadecimal(''))
        self.assertFalse(is_hexadecimal('.'))
        self.assertFalse(is_hexadecimal('..'))
        self.assertFalse(is_hexadecimal('AA/'))
        self.assertFalse(is_hexadecimal('A A'))
        self.assertFalse(is_hexadecimal('abcdefg'))
        self.assertFalse(is_hexadecimal('0xa2h'))
        self.assertFalse(is_hexadecimal('0xa20x'))
        self.assertFalse(is_hexadecimal('0x0123456789abcDEFq'))
        self.assertFalse(is_hexadecimal('0hfedCBA9876543210q'))
        self.assertFalse(is_hexadecimal('01234q56789abcDEF'))
        print('OK - test_invalid_hexadecimal')



from .utils.assert_string import assert_string

rgb_color = "^rgb\((([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5]),){2}([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])\)$"
rgba_color = "^rgba\((([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5]),){3}(0?\.\d|1(\.0)?|0(\.0)?)\)$"
rgb_color_percent = "^rgb\((([0-9]%|[1-9][0-9]%|100%),){2}([0-9]%|[1-9][0-9]%|100%)\)"
rgba_color_percent = "^rgba\((([0-9]%|[1-9][0-9]%|100%),){3}(0?\.\d|1(\.0)?|0(\.0)?)\)"

def is_rgb_color(input: str, include_percent_values: bool = True) -> bool:
    input = assert_string(input)

    is_valid_no_percent_color: bool = input.match(rgb_color) or input.match(rgba_color)

    if not include_percent_values:
        return is_valid_no_percent_color

    is_valid_percent_color: bool = input.match(rgb_color_percent) or input.match(rgba_color_percent)

    return is_valid_no_percent_color or is_valid_percent_color



from typing import TypedDict

from .utils.Classes.String import String
from .utils.assert_string import assert_string
from .utils.merge import merge

class _IsStrongPasswordOptions(TypedDict):
    min_length: int
    min_uppercase: int
    min_lowercase: int
    min_numbers: int
    min_symbols: int
    return_score: bool
    points_per_unique: int
    points_per_repeat: float
    points_for_containing_upper: int
    points_for_containing_lower: int
    points_for_containing_number: int
    points_for_containing_symbol: int

class _Analysis(TypedDict):
    length: int
    unique_chars: int
    uppercase_count: int
    lowercase_count: int
    number_count: int
    symbol_count: int

upper_case_regex = r"^[A-Z]$"
lower_case_regex = r"^[a-z]$"
number_regex = r"^[0-9]$"
symbol_regex = r"^[-#!$@%^&*()_+|~=`{}\[\]:\";'<>?,.\/ ]$"

default_options: _IsStrongPasswordOptions = {
  "min_length": 8,
  "min_uppercase": 1,
  "min_lowercase": 1,
  "min_numbers": 1,
  "min_symbols": 1,
  "return_score": False,
  "points_per_unique": 1,
  "points_per_repeat": 0.5,
  "points_for_containing_lower": 10,
  "points_for_containing_upper": 10,
  "points_for_containing_number": 10,
  "points_for_containing_symbol": 10,
}

def count_chars(pw: String):
    result = {}
    for char in pw:
        if char in result:
            result[char] += result[char] + 1
        else:
            result[char] = 1
    return result

def analyze_password(pw: String) -> _Analysis:
    char_map = count_chars(pw)
    analysis: _Analysis = {
        "length": pw.length,
        "unique_chars": len([*char_map]),
        "uppercase_count": 0,
        "lowercase_count": 0,
        "number_count": 0,
        "symbol_count": 0,
    }
    for char in [*char_map]:
        char = String(char)
        if char.match(upper_case_regex):
            analysis["uppercase_count"] += char_map[char]
        elif char.match(lower_case_regex):
            analysis["lowercase_count"] += char_map[char]
        elif char.match(number_regex):
            analysis["number_count"] += char_map[char]
        elif char.match(symbol_regex):
            analysis["symbol_count"] += char_map[char]
    return analysis

def score_password(analysis: _Analysis, options: _IsStrongPasswordOptions):
    points = 0
    points += analysis["unique_chars"] * options["points_per_unique"]
    points += (analysis["length"] - analysis["unique_chars"]) * options["points_per_unique"]
    if analysis["uppercase_count"] > 0:
        points += options["points_for_containing_upper"]
    if analysis["lowercase_count"] > 0:
        points += options["points_for_containing_lower"]
    if analysis["number_count"] > 0:
        points += options["points_for_containing_number"]
    if analysis["symbol_count"] > 0:
        points += options["points_for_containing_symbol"]
    return points

def is_strong_password(input: str, options: _IsStrongPasswordOptions = {}) -> bool:
    input = assert_string(input)
    options = merge(options, default_options)

    analysis = analyze_password(input)
    if options["return_score"]:
        return score_password(analysis, options)
    return (
        analysis["length"] >= options["min_length"] and
        analysis["uppercase_count"] >= options["min_uppercase"] and
        analysis["lowercase_count"] >= options["min_lowercase"] and
        analysis["number_count"] >= options["min_numbers"] and
        analysis["symbol_count"] >= options["min_symbols"]
    )



import unittest
from pyvalidator import *

class TestIsIp(unittest.TestCase):

    def test_valid_ip(self) -> bool:
        self.assertTrue(is_ip('127.0.0.1'))
        self.assertTrue(is_ip('0.0.0.0'))
        self.assertTrue(is_ip('255.255.255.255'))
        self.assertTrue(is_ip('1.2.3.4'))
        self.assertTrue(is_ip('::1'))
        self.assertTrue(is_ip('2001:db8:0000:1:1:1:1:1'))
        self.assertTrue(is_ip('2001:db8:3:4::192.0.2.33'))
        self.assertTrue(is_ip('2001:41d0:2:a141::1'))
        self.assertTrue(is_ip('::ffff:127.0.0.1'))
        self.assertTrue(is_ip('::0000'))
        self.assertTrue(is_ip('0000::'))
        self.assertTrue(is_ip('1::'))
        self.assertTrue(is_ip('1111:1:1:1:1:1:1:1'))
        self.assertTrue(is_ip('fe80::a6db:30ff:fe98:e946'))
        self.assertTrue(is_ip('::'))
        self.assertTrue(is_ip('::8'))
        self.assertTrue(is_ip('::ffff:127.0.0.1'))
        self.assertTrue(is_ip('::ffff:255.255.255.255'))
        self.assertTrue(is_ip('::ffff:0:255.255.255.255'))
        self.assertTrue(is_ip('::2:3:4:5:6:7:8'))
        self.assertTrue(is_ip('::255.255.255.255'))
        self.assertTrue(is_ip('0:0:0:0:0:ffff:127.0.0.1'))
        self.assertTrue(is_ip('1:2:3:4:5:6:7::'))
        self.assertTrue(is_ip('1:2:3:4:5:6::8'))
        self.assertTrue(is_ip('1::7:8'))
        self.assertTrue(is_ip('1:2:3:4:5::7:8'))
        self.assertTrue(is_ip('1:2:3:4:5::8'))
        self.assertTrue(is_ip('1::6:7:8'))
        self.assertTrue(is_ip('1:2:3:4::6:7:8'))
        self.assertTrue(is_ip('1:2:3:4::8'))
        self.assertTrue(is_ip('1::5:6:7:8'))
        self.assertTrue(is_ip('1:2:3::5:6:7:8'))
        self.assertTrue(is_ip('1:2:3::8'))
        self.assertTrue(is_ip('1::4:5:6:7:8'))
        self.assertTrue(is_ip('1:2::4:5:6:7:8'))
        self.assertTrue(is_ip('1:2::8'))
        self.assertTrue(is_ip('1::3:4:5:6:7:8'))
        self.assertTrue(is_ip('1::8'))
        self.assertTrue(is_ip('fe80::7:8%eth0'))
        self.assertTrue(is_ip('fe80::7:8%1'))
        self.assertTrue(is_ip('64:ff9b::192.0.2.33'))
        self.assertTrue(is_ip('0:0:0:0:0:0:10.0.0.1'))
        print('OK - test_valid_ip')

    def test_invalid_ip(self):
        self.assertFalse(is_ip('abc'))
        self.assertFalse(is_ip('256.0.0.0'))
        self.assertFalse(is_ip('0.0.0.256'))
        self.assertFalse(is_ip('26.0.0.256'))
        self.assertFalse(is_ip('0200.200.200.200'))
        self.assertFalse(is_ip('200.0200.200.200'))
        self.assertFalse(is_ip('200.200.0200.200'))
        self.assertFalse(is_ip('200.200.200.0200'))
        self.assertFalse(is_ip('::banana'))
        self.assertFalse(is_ip('banana::'))
        self.assertFalse(is_ip('::1banana'))
        self.assertFalse(is_ip('::1::'))
        self.assertFalse(is_ip('1:'))
        self.assertFalse(is_ip(':1'))
        self.assertFalse(is_ip(':1:1:1::2'))
        self.assertFalse(is_ip('1:1:1:1:1:1:1:1:1:1:1:1:1:1:1:1'))
        self.assertFalse(is_ip('::11111'))
        self.assertFalse(is_ip('11111:1:1:1:1:1:1:1'))
        self.assertFalse(is_ip('2001:db8:0000:1:1:1:1::1'))
        self.assertFalse(is_ip('0:0:0:0:0:0:ffff:127.0.0.1'))
        self.assertFalse(is_ip('0:0:0:0:ffff:127.0.0.1'))
        print('OK - test_invalid_ip')

    def test_valid_ip_v4_num(self) -> bool:
        self.assertTrue(is_ip('127.0.0.1', 4))
        self.assertTrue(is_ip('0.0.0.0', 4))
        self.assertTrue(is_ip('255.255.255.255', 4))
        self.assertTrue(is_ip('1.2.3.4', 4))
        self.assertTrue(is_ip('255.0.0.1', 4))
        self.assertTrue(is_ip('0.0.1.1', 4))
        print('OK - test_valid_ip_v4_num')

    def test_invalid_ip_v4_num(self):
        self.assertFalse(is_ip('::1', 4))
        self.assertFalse(is_ip('2001:db8:0000:1:1:1:1:1', 4))
        self.assertFalse(is_ip('::ffff:127.0.0.1', 4))
        self.assertFalse(is_ip('137.132.10.01', 4))
        self.assertFalse(is_ip('0.256.0.256', 4))
        self.assertFalse(is_ip('255.256.255.256', 4))
        print('OK - test_invalid_ip_v4_num')

    def test_valid_ip_v4_str(self) -> bool:
        self.assertTrue(is_ip('127.0.0.1', "4"))
        self.assertTrue(is_ip('0.0.0.0', "4"))
        self.assertTrue(is_ip('255.255.255.255', "4"))
        self.assertTrue(is_ip('1.2.3.4', "4"))
        self.assertTrue(is_ip('255.0.0.1', "4"))
        self.assertTrue(is_ip('0.0.1.1', "4"))
        print('OK - test_valid_ip_v4_str')

    def test_invalid_ip_v4_str(self):
        self.assertFalse(is_ip('::1', "4"))
        self.assertFalse(is_ip('2001:db8:0000:1:1:1:1:1', "4"))
        self.assertFalse(is_ip('::ffff:127.0.0.1', "4"))
        self.assertFalse(is_ip('137.132.10.01', "4"))
        self.assertFalse(is_ip('0.256.0.256', "4"))
        self.assertFalse(is_ip('255.256.255.256', "4"))
        print('OK - test_invalid_ip_v4_str')

    def test_valid_ip_v6_num(self) -> bool:
        self.assertTrue(is_ip('::1', 6))
        self.assertTrue(is_ip('2001:db8:0000:1:1:1:1:1', 6))
        self.assertTrue(is_ip('::ffff:127.0.0.1', 6))
        self.assertTrue(is_ip('fe80::1234%1', 6))
        self.assertTrue(is_ip('ff08::9abc%10', 6))
        self.assertTrue(is_ip('ff08::9abc%interface10', 6))
        self.assertTrue(is_ip('ff02::5678%pvc1.3', 6))
        print('OK - test_valid_ip_v6_num')

    def test_invalid_ip_v6_num(self):
        self.assertFalse(is_ip('127.0.0.1', 6))
        self.assertFalse(is_ip('0.0.0.0', 6))
        self.assertFalse(is_ip('255.255.255.255', 6))
        self.assertFalse(is_ip('1.2.3.4', 6))
        self.assertFalse(is_ip('::ffff:287.0.0.1', 6))
        self.assertFalse(is_ip('%', 6))
        self.assertFalse(is_ip('fe80::1234%', 6))
        self.assertFalse(is_ip('fe80::1234%1%3%4', 6))
        self.assertFalse(is_ip('fe80%fe80%', 6))
        print('OK - test_invalid_ip_v6_num')

    def test_valid_ip_v6_str(self) -> bool:
        self.assertTrue(is_ip('::1', "6"))
        self.assertTrue(is_ip('2001:db8:0000:1:1:1:1:1', "6"))
        self.assertTrue(is_ip('::ffff:127.0.0.1', "6"))
        self.assertTrue(is_ip('fe80::1234%1', "6"))
        self.assertTrue(is_ip('ff08::9abc%10', "6"))
        self.assertTrue(is_ip('ff08::9abc%interface10', "6"))
        self.assertTrue(is_ip('ff02::5678%pvc1.3', "6"))
        print('OK - test_valid_ip_v6_str')

    def test_invalid_ip_v6_str(self):
        self.assertFalse(is_ip('127.0.0.1', "6"))
        self.assertFalse(is_ip('0.0.0.0', "6"))
        self.assertFalse(is_ip('255.255.255.255', "6"))
        self.assertFalse(is_ip('1.2.3.4', "6"))
        self.assertFalse(is_ip('::ffff:287.0.0.1', "6"))
        self.assertFalse(is_ip('%', "6"))
        self.assertFalse(is_ip('fe80::1234%', "6"))
        self.assertFalse(is_ip('fe80::1234%1%3%4', "6"))
        self.assertFalse(is_ip('fe80%fe80%', "6"))
        print('OK - test_invalid_ip_v6_str')



import unittest
from pyvalidator import *

class TestIsBase58(unittest.TestCase):

    def test_valid_base64(self):
        self.assertTrue(is_base64(''))
        self.assertTrue(is_base64('Zg=='))
        self.assertTrue(is_base64('Zm8='))
        self.assertTrue(is_base64('Zm9v'))
        self.assertTrue(is_base64('Zm9vYg=='))
        self.assertTrue(is_base64('Zm9vYmE='))
        self.assertTrue(is_base64('Zm9vYmFy'))
        self.assertTrue(is_base64('TG9yZW0gaXBzdW0gZG9sb3Igc2l0IGFtZXQsIGNvbnNlY3RldHVyIGFkaXBpc2NpbmcgZWxpdC4='))
        self.assertTrue(is_base64('Vml2YW11cyBmZXJtZW50dW0gc2VtcGVyIHBvcnRhLg=='))
        self.assertTrue(is_base64('U3VzcGVuZGlzc2UgbGVjdHVzIGxlbw=='))
        self.assertTrue(is_base64('MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAuMPNS1Ufof9EW/M98FNw'))
        self.assertTrue(is_base64('UAKrwflsqVxaxQjBQnHQmiI7Vac40t8x7pIb8gLGV6wL7sBTJiPovJ0V7y7oc0Ye'))
        self.assertTrue(is_base64('rhKh0Rm4skP2z/jHwwZICgGzBvA0rH8xlhUiTvcwDCJ0kc+fh35hNt8srZQM4619'))
        self.assertTrue(is_base64('FTgB66Xmp4EtVyhpQV+t02g6NzK72oZI0vnAvqhpkxLeLiMCyrI416wHm5Tkukhx'))
        self.assertTrue(is_base64('QmcL2a6hNOyu0ixX/x2kSFXApEnVrJ+/IxGyfyw8kf4N2IZpW5nEP847lpfj0SZZ'))
        self.assertTrue(is_base64('Fwrd1mnfnDbYohX2zRptLy2ZUn06Qo9pkG5ntvFEPo9bfZeULtjYzIl6K8gJ2uGZ'))
        self.assertTrue(is_base64('HQIDAQAB'))
        print('OK - test_valid_base64')

    def test_invalid_base64(self):
        self.assertFalse(is_base64('12345'))
        self.assertFalse(is_base64('Vml2YW11cyBmZXJtZtesting123'))
        self.assertFalse(is_base64('Zg='))
        self.assertFalse(is_base64('Z==='))
        self.assertFalse(is_base64('Zm=8'))
        self.assertFalse(is_base64('=m9vYg=='))
        self.assertFalse(is_base64('Zm9vYmFy===='))
        print('OK - test_invalid_base64')

    def test_valid_base64_with_url_safe_flag(self):
        self.assertTrue(is_base64('', { 'url_safe': True }))
        self.assertTrue(is_base64('bGFkaWVzIGFuZCBnZW50bGVtZW4sIHdlIGFyZSBmbG9hdGluZyBpbiBzcGFjZQ', { 'url_safe': True }))
        self.assertTrue(is_base64('1234', { 'url_safe': True }))
        self.assertTrue(is_base64('bXVtLW5ldmVyLXByb3Vk', { 'url_safe': True }))
        self.assertTrue(is_base64('PDw_Pz8-Pg', { 'url_safe': True }))
        self.assertTrue(is_base64('VGhpcyBpcyBhbiBlbmNvZGVkIHN0cmluZw', { 'url_safe': True }))
        print('OK - test_valid_base64_with_url_safe_flag')

    def test_invalid_base64_with_url_safe_flag(self):
        self.assertFalse(is_base64(' AA', { 'url_safe': True }))
        self.assertFalse(is_base64('\tAA', { 'url_safe': True }))
        self.assertFalse(is_base64('\rAA', { 'url_safe': True }))
        self.assertFalse(is_base64('\nAA', { 'url_safe': True }))
        self.assertFalse(is_base64('This+isa/bad+base64Url==', { 'url_safe': True }))
        self.assertFalse(is_base64('0K3RgtC+INC30LDQutC+0LTQuNGA0L7QstCw0L3QvdCw0Y8g0YHRgtGA0L7QutCw', { 'url_safe': True }))
        print('OK - test_invalid_base64_with_url_safe_flag')



import unittest
from pyvalidator import *

class TestIsJwt(unittest.TestCase):

    def test_valid_jwt(self):
        self.assertTrue(is_jwt('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJsb2dnZWRJbkFzIjoiYWRtaW4iLCJpYXQiOjE0MjI3Nzk2Mzh9.gzSraSYS8EXBxLN_oWnFSRgCzcmJmMjLiuyu5CSpyHI'))
        self.assertTrue(is_jwt('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJsb3JlbSI6Imlwc3VtIn0.ymiJSsMJXR6tMSr8G9usjQ15_8hKPDv_CArLhxw28MI'))
        self.assertTrue(is_jwt('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJkb2xvciI6InNpdCIsImFtZXQiOlsibG9yZW0iLCJpcHN1bSJdfQ.rRpe04zbWbbJjwM43VnHzAboDzszJtGrNsUxaqQ-GQ8'))
        self.assertTrue(is_jwt('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqb2huIjp7ImFnZSI6MjUsImhlaWdodCI6MTg1fSwiamFrZSI6eyJhZ2UiOjMwLCJoZWlnaHQiOjI3MH19.YRLPARDmhGMC3BBk_OhtwwK21PIkVCqQe8ncIRPKo-E'))
        self.assertTrue(is_jwt('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2MjM5MDIyfQ'))
        print('OK - test_valid_jwt')

    def test_invalid_jwt(self):
        self.assertFalse(is_jwt('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9'))
        self.assertFalse(is_jwt('$Zs.ewu.su84'))
        self.assertFalse(is_jwt('ks64$S/9.dy$§kz.3sd73b'))
        print('OK - test_invalid_jwt')

    def test_fail_jwt(self):
        self.assertRaises(Exception, is_jwt, None)
        self.assertRaises(Exception, is_jwt, {})



import unittest
from pyvalidator import *

class TestIsPort(unittest.TestCase):

    def test_valid_semantic_version(self):
        self.assertTrue(is_semantic_version('0.0.4'))
        self.assertTrue(is_semantic_version('1.2.3'))
        self.assertTrue(is_semantic_version('10.20.30'))
        self.assertTrue(is_semantic_version('1.1.2-prerelease+meta'))
        self.assertTrue(is_semantic_version('1.1.2+meta'))
        self.assertTrue(is_semantic_version('1.1.2+meta-valid'))
        self.assertTrue(is_semantic_version('1.0.0-alpha'))
        self.assertTrue(is_semantic_version('1.0.0-beta'))
        self.assertTrue(is_semantic_version('1.0.0-alpha.beta'))
        self.assertTrue(is_semantic_version('1.0.0-alpha.beta.1'))
        self.assertTrue(is_semantic_version('1.0.0-alpha.1'))
        self.assertTrue(is_semantic_version('1.0.0-alpha0.valid'))
        self.assertTrue(is_semantic_version('1.0.0-alpha.0valid'))
        self.assertTrue(is_semantic_version('1.0.0-alpha-a.b-c-somethinglong+build.1-aef.1-its-okay'))
        self.assertTrue(is_semantic_version('1.0.0-rc.1+build.1'))
        self.assertTrue(is_semantic_version('2.0.0-rc.1+build.123'))
        self.assertTrue(is_semantic_version('1.2.3-beta'))
        self.assertTrue(is_semantic_version('10.2.3-DEV-SNAPSHOT'))
        self.assertTrue(is_semantic_version('1.2.3-SNAPSHOT-123'))
        self.assertTrue(is_semantic_version('1.0.0'))
        self.assertTrue(is_semantic_version('2.0.0'))
        self.assertTrue(is_semantic_version('1.1.7'))
        self.assertTrue(is_semantic_version('2.0.0+build.1848'))
        self.assertTrue(is_semantic_version('2.0.1-alpha.1227'))
        self.assertTrue(is_semantic_version('1.0.0-alpha+beta'))
        self.assertTrue(is_semantic_version('1.2.3----RC-SNAPSHOT.12.9.1--.12+788'))
        self.assertTrue(is_semantic_version('1.2.3----R-S.12.9.1--.12+meta'))
        self.assertTrue(is_semantic_version('1.2.3----RC-SNAPSHOT.12.9.1--.12'))
        self.assertTrue(is_semantic_version('1.0.0+0.build.1-rc.10000aaa-kk-0.1'))
        self.assertTrue(is_semantic_version('99999999999999999999999.999999999999999999.99999999999999999'))
        self.assertTrue(is_semantic_version('1.0.0-0A.is.legal'))
        print('OK - test_valid_semantic_version')

    def test_invalid_semantic_version(self):
        self.assertFalse(is_semantic_version(''))
        self.assertFalse(is_semantic_version('...'))
        self.assertFalse(is_semantic_version('-invalid+invalid'))
        self.assertFalse(is_semantic_version('-invalid.01'))
        self.assertFalse(is_semantic_version('alpha'))
        self.assertFalse(is_semantic_version('alpha.beta'))
        self.assertFalse(is_semantic_version('alpha.beta.1'))
        self.assertFalse(is_semantic_version('alpha.1'))
        self.assertFalse(is_semantic_version('alpha+beta'))
        self.assertFalse(is_semantic_version('alpha_beta'))
        self.assertFalse(is_semantic_version('alpha.'))
        self.assertFalse(is_semantic_version('alpha..'))
        self.assertFalse(is_semantic_version('beta'))
        self.assertFalse(is_semantic_version('1.0.0-alpha_beta'))
        self.assertFalse(is_semantic_version('-alpha.'))
        self.assertFalse(is_semantic_version('1.0.0-alpha..'))
        self.assertFalse(is_semantic_version('1.0.0-alpha..1'))
        self.assertFalse(is_semantic_version('1.0.0-alpha...1'))
        self.assertFalse(is_semantic_version('1.0.0-alpha....1'))
        self.assertFalse(is_semantic_version('1.0.0-alpha.....1'))
        self.assertFalse(is_semantic_version('1.0.0-alpha......1'))
        self.assertFalse(is_semantic_version('1.0.0-alpha.......1'))
        self.assertFalse(is_semantic_version('01.1.1'))
        self.assertFalse(is_semantic_version('1.01.1'))
        self.assertFalse(is_semantic_version('1.1.01'))
        self.assertFalse(is_semantic_version('1.2'))
        self.assertFalse(is_semantic_version('1.2.3.DEV'))
        self.assertFalse(is_semantic_version('1.2-SNAPSHOT'))
        self.assertFalse(is_semantic_version('1.2.31.2.3----RC-SNAPSHOT.12.09.1--..12+788'))
        self.assertFalse(is_semantic_version('1.2-RC-SNAPSHOT'))
        self.assertFalse(is_semantic_version('-1.0.3-gamma+b7718'))
        self.assertFalse(is_semantic_version('+justmeta'))
        self.assertFalse(is_semantic_version('9.8.7+meta+meta'))
        self.assertFalse(is_semantic_version('9.8.7-whatever+meta+meta'))
        self.assertFalse(is_semantic_version('99999999999999999999999.999999999999999999.99999999999999999-'))
        self.assertFalse(is_semantic_version('--SSS.RC-UPDATE.13.19.2---------..13'))
        print('OK - test_invalid_semantic_version')



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.slice import slice_and_upper_case
from .is_ISO31661_alpha2 import country_codes

is_bic_reg = RegEx('^[A-Za-z]{6}[A-Za-z0-9]{2}([A-Za-z0-9]{3})?$')

def is_bic(input: str) -> bool:
    input = assert_string(input)

    if not slice_and_upper_case(input, 4, 6) in country_codes:
        return False

    return is_bic_reg.match(input)



import unittest
from pyvalidator import *

class TestIsOctal(unittest.TestCase):

    def test_valid_octal(self):
        self.assertTrue(is_octal('076543210'))
        self.assertTrue(is_octal('0o01234567'))
        print('OK - test_valid_octal')

    def test_invalid_octal(self):
        self.assertFalse(is_octal(''))
        self.assertFalse(is_octal('.'))
        self.assertFalse(is_octal('/'))
        self.assertFalse(is_octal('999'))
        self.assertFalse(is_octal('abcdefg'))
        self.assertFalse(is_octal('012345678'))
        self.assertFalse(is_octal('012345670c'))
        self.assertFalse(is_octal('00c12345670c'))
        print('OK - test_invalid_octal')



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.assert_string import assert_string

base32 = RegEx("^[A-Z2-7]+=*$")

def is_base32(input: str) -> bool:
    input = assert_string(input)
    return bool(input.length % 8 == 0 and base32.match(input))



import unittest
from pyvalidator import *

class TestIsBic(unittest.TestCase):

    def test_valid_bic_codes(self):
        self.assertTrue(is_bic('SBICKEN1345'))
        self.assertTrue(is_bic('SBICKEN1'))
        self.assertTrue(is_bic('SBICKENY'))
        self.assertTrue(is_bic('SBICKEN1YYP'))
        print('OK - test_valid_bic_codes')

    def test_invalid_bic_codes(self):
        self.assertFalse(is_bic('SBIC23NXXX'))
        self.assertFalse(is_bic('S23CKENXXXX'))
        self.assertFalse(is_bic('SBICKENXX'))
        self.assertFalse(is_bic('SBICKENXX9'))
        self.assertFalse(is_bic('SBICKEN13458'))
        self.assertFalse(is_bic('SBICKEN'))
        print('OK - test_invalid_bic_codes')



import unittest
from pyvalidator import *

class TestIsByteLength(unittest.TestCase):

    def test_valid_byte_length_min_2(self):
        self.assertTrue(is_byte_length('abc', { "min": 2 }))
        self.assertTrue(is_byte_length('de', { "min": 2 }))
        self.assertTrue(is_byte_length('abcd', { "min": 2 }))
        self.assertTrue(is_byte_length('ｇｍａｉｌ', { "min": 2 }))
        print('OK - test_valid_byte_length')

    def test_invalid_byte_length_min_2(self):
        self.assertFalse(is_byte_length('', { "min": 2 }))
        self.assertFalse(is_byte_length('a', { "min": 2 }))
        print('OK - test_invalid_byte_length')

    def test_valid_byte_length_min_2_max_3(self):
        self.assertTrue(is_byte_length('abc', { "min": 2, "max": 3 }))
        self.assertTrue(is_byte_length('de', { "min": 2, "max": 3 }))
        self.assertTrue(is_byte_length('ｇ', { "min": 2, "max": 3 }))
        print('OK - test_valid_byte_length')

    def test_invalid_byte_length_min_2_max_3(self):
        self.assertFalse(is_byte_length('', { "min": 2, "max": 3 }))
        self.assertFalse(is_byte_length('a', { "min": 2, "max": 3 }))
        self.assertFalse(is_byte_length('abcd', { "min": 2, "max": 3 }))
        self.assertFalse(is_byte_length('ｇｍ', { "min": 2, "max": 3 }))
        print('OK - test_invalid_byte_length')

    def test_valid_byte_length_max_0(self):
        self.assertTrue(is_byte_length('', { "max": 0 }))
        print('OK - test_valid_byte_length')

    def test_invalid_byte_length_max_0(self):
        self.assertFalse(is_byte_length('g', { "max": 0 }))
        self.assertFalse(is_byte_length('a', { "max": 0 }))
        print('OK - test_invalid_byte_length')



import unittest
from pyvalidator import *
import datetime

class TestIsBefore(unittest.TestCase):

    def test_valid_before_dates_against_a_start_date(self):
        self.assertTrue(is_before('2010-07-02', '08/04/2011'))
        self.assertTrue(is_before('2010-08-04', '08/04/2011'))
        self.assertTrue(is_before(datetime.datetime.utcfromtimestamp(0).isoformat(), '08/04/2011'))
        print('OK - test_valid_before_dates_against_a_start_date')

    def test_invalid_before_dates_against_a_start_date(self):
        self.assertFalse(is_before('08/04/2011', '08/04/2011'))
        self.assertFalse(is_before(datetime.datetime(2011, 9, 10).isoformat(), '08/04/2011'))
        print('OK - test_invalid_before_dates_against_a_start_date')



import unittest
from pyvalidator import *

class TestIsHash(unittest.TestCase):

    len_32 = [
        'd94f3f016ae679c3008de268209132f2',
        '751adbc511ccbe8edf23d486fa4581cd',
        '88dae00e614d8f24cfd5a8b3f8002e93',
        '0bf1c35032a71a14c2f719e5a14c1e96',
        'd94f3F016Ae679C3008de268209132F2',
        '88DAE00e614d8f24cfd5a8b3f8002E93',
    ]

    len_8 = [
        'd94f3f01',
        '751adbc5',
        '88dae00e',
        '0bf1c350',
        '88DAE00e',
        '751aDBc5',
    ]

    len_40 = [
        '3ca25ae354e192b26879f651a51d92aa8a34d8d3',
        'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d',
        'beb8c3f30da46be179b8df5f5ecb5e4b10508230',
        'efd5d3b190e893ed317f38da2420d63b7ae0d5ed',
        'AAF4c61ddCC5e8a2dabede0f3b482cd9AEA9434D',
        '3ca25AE354e192b26879f651A51d92aa8a34d8D3',
    ]

    len_48 = [
        '6281a1f098c5e7290927ed09150d43ff3990a0fe1a48267c',
        '56268f7bc269cf1bc83d3ce42e07a85632394737918f4760',
        '46fc0125a148788a3ac1d649566fc04eb84a746f1a6e4fa7',
        '7731ea1621ae99ea3197b94583d034fdbaa4dce31a67404a',
        '6281A1f098c5e7290927ed09150d43ff3990a0fe1a48267C',
        '46FC0125a148788a3AC1d649566fc04eb84A746f1a6E4fa7',
    ]

    len_64 = [
        '2cf24dba5fb0a30e26e83b2ac5b9e29e1b161e5c1fa7425e73043362938b9824',
        '1d996e033d612d9af2b44b70061ee0e868bfd14c2dd90b129e1edeb7953e7985',
        '80f70bfeaed5886e33536bcfa8c05c60afef5a0e48f699a7912d5e399cdcc441',
        '579282cfb65ca1f109b78536effaf621b853c9f7079664a3fbe2b519f435898c',
        '2CF24dba5FB0a30e26E83b2AC5b9E29E1b161e5C1fa7425E73043362938b9824',
        '80F70bFEAed5886e33536bcfa8c05c60aFEF5a0e48f699a7912d5e399cdCC441',
    ]

    len_96 = [
        '3fed1f814d28dc5d63e313f8a601ecc4836d1662a19365cbdcf6870f6b56388850b58043f7ebf2418abb8f39c3a42e31',
        'b330f4e575db6e73500bd3b805db1a84b5a034e5d21f0041d91eec85af1dfcb13e40bb1c4d36a72487e048ac6af74b58',
        'bf547c3fc5841a377eb1519c2890344dbab15c40ae4150b4b34443d2212e5b04aa9d58865bf03d8ae27840fef430b891',
        'fc09a3d11368386530f985dacddd026ae1e44e0e297c805c3429d50744e6237eb4417c20ffca8807b071823af13a3f65',
        '3fed1f814d28dc5d63e313f8A601ecc4836d1662a19365CBDCf6870f6b56388850b58043f7ebf2418abb8f39c3a42e31',
        'b330f4E575db6e73500bd3b805db1a84b5a034e5d21f0041d91EEC85af1dfcb13e40bb1c4d36a72487e048ac6af74b58',
    ]

    len_128 = [
        '9b71d224bd62f3785d96d46ad3ea3d73319bfbc2890caadae2dff72519673ca72323c3d99ba5c11d7c7acc6e14b8c5da0c4663475c2e5c3adef46f73bcdec043',
        '83c586381bf5ba94c8d9ba8b6b92beb0997d76c257708742a6c26d1b7cbb9269af92d527419d5b8475f2bb6686d2f92a6649b7f174c1d8306eb335e585ab5049',
        '45bc5fa8cb45ee408c04b6269e9f1e1c17090c5ce26ffeeda2af097735b29953ce547e40ff3ad0d120e5361cc5f9cee35ea91ecd4077f3f589b4d439168f91b9',
        '432ac3d29e4f18c7f604f7c3c96369a6c5c61fc09bf77880548239baffd61636d42ed374f41c261e424d20d98e320e812a6d52865be059745fdb2cb20acff0ab',
        '9B71D224bd62f3785D96d46ad3ea3d73319bFBC2890CAAdae2dff72519673CA72323C3d99ba5c11d7c7ACC6e14b8c5DA0c4663475c2E5c3adef46f73bcDEC043',
        '432AC3d29E4f18c7F604f7c3c96369A6C5c61fC09Bf77880548239baffd61636d42ed374f41c261e424d20d98e320e812a6d52865be059745fdb2cb20acff0ab',
    ]

    def test_valid_hash_md5(self):
        for hash in self.len_32:
            self.assertTrue(is_hash(hash, 'md5'))
        print('OK - test_valid_hash_md5')

    def test_invalid_hash_md5(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'md5'))
        self.assertFalse(is_hash('q94375dj93458w34', 'md5'))
        self.assertFalse(is_hash('q94375dj93458w34', 'md5'))
        self.assertFalse(is_hash('39485729348', 'md5'))
        self.assertFalse(is_hash('q943', 'md5'))
        self.assertFalse(is_hash('%&FHKJFvk', 'md5'))
        self.assertFalse(is_hash('39485729348', 'md5'))
        print('OK - test_invalid_hash_md5')

    def test_valid_hash_md4(self):
        for hash in self.len_32:
            self.assertTrue(is_hash(hash, 'md4'))
        print('OK - test_valid_hash_md4')

    def test_invalid_hash_md4(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'md4'))
        self.assertFalse(is_hash('q94375dj93458w34', 'md4'))
        self.assertFalse(is_hash('q94375dj93458w34', 'md4'))
        self.assertFalse(is_hash('39485729348', 'md4'))
        self.assertFalse(is_hash('q943', 'md4'))
        self.assertFalse(is_hash('%&FHKJFvk', 'md4'))
        self.assertFalse(is_hash('39485729348', 'md4'))
        print('OK - test_invalid_hash_md4')

    def test_valid_hash_ripemd128(self):
        for hash in self.len_32:
            self.assertTrue(is_hash(hash, 'ripemd128'))
        print('OK - test_valid_hash_ripemd128')

    def test_invalid_hash_ripemd128(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'ripemd128'))
        self.assertFalse(is_hash('q94375dj93458w34', 'ripemd128'))
        self.assertFalse(is_hash('q94375dj93458w34', 'ripemd128'))
        self.assertFalse(is_hash('39485729348', 'ripemd128'))
        self.assertFalse(is_hash('q943', 'ripemd128'))
        self.assertFalse(is_hash('%&FHKJFvk', 'ripemd128'))
        self.assertFalse(is_hash('39485729348', 'ripemd128'))
        print('OK - test_invalid_hash_ripemd128')

    def test_valid_hash_tiger128(self):
        for hash in self.len_32:
            self.assertTrue(is_hash(hash, 'tiger128'))
        print('OK - test_valid_hash_tiger128')

    def test_invalid_hash_tiger128(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'tiger128'))
        self.assertFalse(is_hash('q94375dj93458w34', 'tiger128'))
        self.assertFalse(is_hash('q94375dj93458w34', 'tiger128'))
        self.assertFalse(is_hash('39485729348', 'tiger128'))
        self.assertFalse(is_hash('q943', 'tiger128'))
        self.assertFalse(is_hash('%&FHKJFvk', 'tiger128'))
        self.assertFalse(is_hash('39485729348', 'tiger128'))
        print('OK - test_invalid_hash_tiger128')

    def test_valid_hash_crc32(self):
        for hash in self.len_8:
            self.assertTrue(is_hash(hash, 'crc32'))
        print('OK - test_valid_hash_crc32')

    def test_invalid_hash_crc32(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'crc32'))
        self.assertFalse(is_hash('q94375dj93458w34', 'crc32'))
        self.assertFalse(is_hash('q943', 'crc32'))
        self.assertFalse(is_hash('39485729348', 'crc32'))
        self.assertFalse(is_hash('%&FHKJFvk', 'crc32'))
        print('OK - test_invalid_hash_crc32')

    def test_valid_hash_crc32b(self):
        for hash in self.len_8:
            self.assertTrue(is_hash(hash, 'crc32b'))
        print('OK - test_valid_hash_crc32b')

    def test_invalid_hash_crc32b(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'crc32b'))
        self.assertFalse(is_hash('q94375dj93458w34', 'crc32b'))
        self.assertFalse(is_hash('q943', 'crc32b'))
        self.assertFalse(is_hash('39485729348', 'crc32b'))
        self.assertFalse(is_hash('%&FHKJFvk', 'crc32b'))
        print('OK - test_invalid_hash_crc32b')

    def test_valid_hash_sha1(self):
        for hash in self.len_40:
            self.assertTrue(is_hash(hash, 'sha1'))
        print('OK - test_valid_hash_sha1')

    def test_invalid_hash_sha1(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'sha1'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjk', 'sha1'))
        self.assertFalse(is_hash('q943', 'sha1'))
        self.assertFalse(is_hash('q94375dj93458w34', 'sha1'))
        self.assertFalse(is_hash('%&FHKJFvk', 'sha1'))
        print('OK - test_invalid_hash_sha1')

    def test_valid_hash_tiger160(self):
        for hash in self.len_40:
            self.assertTrue(is_hash(hash, 'tiger160'))
        print('OK - test_valid_hash_tiger160')

    def test_invalid_hash_tiger160(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'tiger160'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjk', 'tiger160'))
        self.assertFalse(is_hash('q943', 'tiger160'))
        self.assertFalse(is_hash('q94375dj93458w34', 'tiger160'))
        self.assertFalse(is_hash('%&FHKJFvk', 'tiger160'))
        print('OK - test_invalid_hash_tiger160')

    def test_valid_hash_sha256(self):
        for hash in self.len_64:
            self.assertTrue(is_hash(hash, 'sha256'))
        print('OK - test_valid_hash_sha256')

    def test_invalid_hash_sha256(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'sha256'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjk', 'sha256'))
        self.assertFalse(is_hash('q943', 'sha256'))
        self.assertFalse(is_hash('q94375dj93458w34', 'sha256'))
        self.assertFalse(is_hash('%&FHKJFvk', 'sha256'))
        print('OK - test_invalid_hash_sha256')

    def test_valid_hash_sha384(self):
        for hash in self.len_96:
            self.assertTrue(is_hash(hash, 'sha384'))
        print('OK - test_valid_hash_sha384')

    def test_invalid_hash_sha384(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'sha384'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjk', 'sha384'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjklololo', 'sha384'))
        self.assertFalse(is_hash('q943', 'sha384'))
        self.assertFalse(is_hash('q94375dj93458w34', 'sha384'))
        self.assertFalse(is_hash('%&FHKJFvk', 'sha384'))
        print('OK - test_invalid_hash_sha384')

    def test_valid_hash_sha512(self):
        for hash in self.len_128:
            self.assertTrue(is_hash(hash, 'sha512'))
        print('OK - test_valid_hash_sha512')

    def test_invalid_hash_sha512(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'sha512'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjk', 'sha512'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjklololo', 'sha512'))
        self.assertFalse(is_hash('q943', 'sha512'))
        self.assertFalse(is_hash('q94375dj93458w34', 'sha512'))
        self.assertFalse(is_hash('%&FHKJFvk', 'sha512'))
        print('OK - test_invalid_hash_sha512')

    def test_valid_hash_tiger192(self):
        for hash in self.len_48:
            self.assertTrue(is_hash(hash, 'tiger192'))
        print('OK - test_valid_hash_tiger192')

    def test_invalid_hash_tiger192(self):
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1', 'tiger192'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjk', 'tiger192'))
        self.assertFalse(is_hash('KYT0bf1c35032a71a14c2f719e5a14c1dsjkjkjkjkkjklololo', 'tiger192'))
        self.assertFalse(is_hash('q943', 'tiger192'))
        self.assertFalse(is_hash('q94375dj93458w34', 'tiger192'))
        self.assertFalse(is_hash('%&FHKJFvk', 'tiger192'))
        print('OK - test_invalid_hash_tiger192')



import unittest
from pyvalidator import *

class TestIsAscii(unittest.TestCase):

    def test_valid_ascii_strings(self):
        self.assertTrue(is_ascii('foobar'))
        self.assertTrue(is_ascii('0987654321'))
        self.assertTrue(is_ascii('test@example.com'))
        self.assertTrue(is_ascii('1234abcDEF'))
        print('OK - valid_ascii_strings')

    def test_invalid_ascii_strings(self):
        self.assertFalse(is_ascii('ｆｏｏbar'))
        self.assertFalse(is_ascii('ｘｙｚ０９８'))
        self.assertFalse(is_ascii('１２３456'))
        self.assertFalse(is_ascii('ｶﾀｶﾅ'))
        print('OK - invalid_ascii_strings')



from typing import TypedDict

from .utils.Classes.RegEx import RegEx
from .utils.Classes.String import String
from .utils.Classes.List import List
from .utils.assert_string import assert_string
from .utils.merge import merge
from .utils.includes import includesSome
from .is_byte_length import is_byte_length
from .is_ip import is_ip
from .is_fqdn import is_fqdn

class IsEmailOptions(TypedDict):
    allow_display_name: bool
    require_display_name: bool
    allow_utf8_local_part: bool
    require_tld: bool
    ignore_max_length: bool
    domain_specific_validation: bool
    allow_ip_domain: bool
    blacklisted_chars: str
    host_blacklist: List

default_email_options: IsEmailOptions = {
  "allow_display_name": False,
  "require_display_name": False,
  "allow_utf8_local_part": True,
  "require_tld": True,
  "ignore_max_length": False,
  "domain_specific_validation": False,
  "allow_ip_domain": False,
  "blacklisted_chars": '',
  "host_blacklist": [],
}

split_name_address = r"^([^\x00-\x1F\x7F-\x9F]+)<"
email_user_part = r"^[a-z\d!#\$%&'\*\+\-\/=\?\^_`{\|}~]+$"
gmail_user_part = r"^[a-z\d]+$"
quoted_email_user = r"^([\s\x01-\x08\x0b\x0c\x0e-\x1f\x7f\x21\x23-\x5b\x5d-\x7e]|(\\[\x01-\x09\x0b\x0c\x0d-\x7f]))*$"
email_user_utf8_part = r"^[a-z\d!#\$%&'\*\+\-\/=\?\^_`{\|}~\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+$"
quoted_email_user_utf8 = r"^([\s\x01-\x08\x0b\x0c\x0e-\x1f\x7f\x21\x23-\x5b\x5d-\x7e\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]|(\\[\x01-\x09\x0b\x0c\x0d-\x7f\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))*$"
default_max_email_length = 254


def validate_display_name(display_name: String):
    display_name_without_quotes = display_name
    if display_name[0] == '"' and display_name[display_name.length - 1] == '"':
        display_name_without_quotes = display_name.substring(1, display_name.length - 1)

    if not display_name_without_quotes.trim():
        return False

    illegal_chars = ['"', '.', ';', '<', '>']
    contains_illegal = includesSome(display_name_without_quotes, illegal_chars)

    if contains_illegal:
        
        if display_name_without_quotes == display_name:
            return False

        all_start_with_back_slash = (
            display_name_without_quotes.split('"').length == display_name_without_quotes.split('\\"').length
        )

        if not all_start_with_back_slash:
            return False

    return True

def is_email(input: str, options: IsEmailOptions = {}) -> bool:
    input = assert_string(input)
    options = merge(options, default_email_options)

    if options["require_display_name"] or options["allow_display_name"]:
        display_email = input.findMatches(split_name_address, 'i')
        if display_email:
            display_name = String(display_email[0])

            input = input.sub(RegEx.escape(display_name), '').sub(r"(^<|>$)", '')
            if display_name.endswith(' '):
                display_name = display_name.substring(0, display_name.length-1)
            
            if not validate_display_name(display_name):
                return False
        elif options["require_display_name"]:
            return False

    if (not options["ignore_max_length"]) and input.length > default_max_email_length:
        return False

    parts = input.split('@')
    domain = String(parts.pop())
    parts = List(parts[:-1])
    lower_domain = domain.lower()

    if List(options["host_blacklist"]).includes(lower_domain):
        return False

    user = String(parts.join('@'))

    if options["domain_specific_validation"] and (lower_domain == 'gmail.com' or lower_domain == 'googlemail.com'):
        user = user.lower()
        username = String(user.split('+')[0])

        if not is_byte_length(username.sub(RegEx('\.', 'g'), ''), { "min": 6, "max": 30 }):
            return False

        user_parts = username.split('.')
        for user_part in user_parts:
            if not String(user_part).match(gmail_user_part):
                return False

    if (
        (not options["ignore_max_length"]) and  (
            not is_byte_length(user, { "max": 64 }) or
            not is_byte_length(domain, { "max": 254 })
        )
    ):
        return False

    if not is_fqdn(domain, { "require_tld": options["require_tld"] }):

        if (not options["allow_ip_domain"]):
            return False

        if not is_ip(domain):
            if not domain.startswith('[') or not domain.endswith(']'):
                return False

            no_bracketdomain = domain.substring(1, domain.length - 2)

            if no_bracketdomain.length == 0 or not is_ip(no_bracketdomain):
                return False

    if user.length and user[0] == '"':
        user = user.slice(1, user.length - 1)
        if options["allow_utf8_local_part"]:
            return user.match(quoted_email_user_utf8, 'i')
        else:
            return user.match(quoted_email_user, 'i')

    pattern = email_user_utf8_part if options["allow_utf8_local_part"] else email_user_part
    user_parts = user.split('.')
    for user_part in user_parts:
        if not String(user_part).match(pattern, 'i'):
                return False

    if options["blacklisted_chars"]:
        if user.findMatches(RegEx("[" + options["blacklisted_chars"] + "]+", 'g')):
            return False

    return True



from pyvalidator.is_after import is_after
from pyvalidator.is_alpha import is_alpha
from pyvalidator.is_ascii import is_ascii
from pyvalidator.is_base32 import is_base32
from pyvalidator.is_base58 import is_base58
from pyvalidator.is_base64 import is_base64
from pyvalidator.is_before import is_before
from pyvalidator.is_BIC import is_bic
from pyvalidator.is_ISO31661_alpha2 import is_ISO31661_alpha2
from pyvalidator.is_boolean import is_boolean
from pyvalidator.is_btc_address import is_btc_address
from pyvalidator.is_byte_length import is_byte_length
from pyvalidator.is_credit_card import is_credit_card
from pyvalidator.is_currency import is_currency
from pyvalidator.is_data_uri import is_data_uri
from pyvalidator.is_decimal import is_decimal
from pyvalidator.is_divisible_by import is_divisible_by
from pyvalidator.is_float import is_float
from pyvalidator.is_int import is_int
from pyvalidator.is_number import is_number
from pyvalidator.is_ean import is_ean
from pyvalidator.is_uuid import is_uuid
from pyvalidator.is_isrc import is_isrc
from pyvalidator.is_jwt import is_jwt
from pyvalidator.is_md5 import is_md5
from pyvalidator.is_port import is_port
from pyvalidator.is_ip import is_ip
from pyvalidator.is_semantic_version import is_semantic_version
from pyvalidator.is_hexadecimal import is_hexadecimal
from pyvalidator.is_mongo_id import is_mongo_id
from pyvalidator.is_uppercase import is_uppercase
from pyvalidator.is_slug import is_slug
from pyvalidator.is_full_width import is_full_width
from pyvalidator.is_rgb_color import is_rgb_color
from pyvalidator.is_octal import is_octal
from pyvalidator.is_passport_number import is_passport_number
from pyvalidator.is_lowercase import is_lowercase
from pyvalidator.is_ip_range import is_ip_range
from pyvalidator.is_json import is_json
from pyvalidator.is_hash import is_hash
from pyvalidator.is_email import is_email
from pyvalidator.is_fqdn import is_fqdn
from pyvalidator.is_imei import is_imei
from pyvalidator.is_strong_password import is_strong_password
from pyvalidator.is_emoji import is_emoji
from pyvalidator.is_mobile_number import is_mobile_number
from pyvalidator.is_date import is_date
from pyvalidator.is_hsl import is_hsl
from pyvalidator.is_ethereum_address import is_ethereum_address
from pyvalidator.is_lat_long import is_lat_long



alpha = {
  'en-US': '^[A-Z]+$',
  'az-AZ': '^[A-VXYZÇƏĞİıÖŞÜ]+$',
  'bg-BG': '^[А-Я]+$',
  'cs-CZ': '^[A-ZÁČĎÉĚÍŇÓŘŠŤÚŮÝŽ]+$',
  'da-DK': '^[A-ZÆØÅ]+$',
  'de-DE': '^[A-ZÄÖÜß]+$',
  'el-GR': '^[Α-ώ]+$',
  'es-ES': '^[A-ZÁÉÍÑÓÚÜ]+$',
  'fa-IR': '^[ابپتثجچحخدذرزژسشصضطظعغفقکگلمنوهی]+$',
  'fr-FR': '^[A-ZÀÂÆÇÉÈÊËÏÎÔŒÙÛÜŸ]+$',
  'it-IT': '^[A-ZÀÉÈÌÎÓÒÙ]+$',
  'nb-NO': '^[A-ZÆØÅ]+$',
  'nl-NL': '^[A-ZÁÉËÏÓÖÜÚ]+$',
  'nn-NO': '^[A-ZÆØÅ]+$',
  'hu-HU': '^[A-ZÁÉÍÓÖŐÚÜŰ]+$',
  'pl-PL': '^[A-ZĄĆĘŚŁŃÓŻŹ]+$',
  'pt-PT': '^[A-ZÃÁÀÂÄÇÉÊËÍÏÕÓÔÖÚÜ]+$',
  'ru-RU': '^[А-ЯЁ]+$',
  'sl-SI': '^[A-ZČĆĐŠŽ]+$',
  'sk-SK': '^[A-ZÁČĎÉÍŇÓŠŤÚÝŽĹŔĽÄÔ]+$',
  'sr-RS@latin': '^[A-ZČĆŽŠĐ]+$',
  'sr-RS': '^[А-ЯЂЈЉЊЋЏ]+$',
  'sv-SE': '^[A-ZÅÄÖ]+$',
  'th-TH': '^[ก-๐\s]+$',
  'tr-TR': '^[A-ZÇĞİıÖŞÜ]+$',
  'uk-UA': '^[А-ЩЬЮЯЄIЇҐі]+$',
  'vi-VN': '^[A-ZÀÁẠẢÃÂẦẤẬẨẪĂẰẮẶẲẴĐÈÉẸẺẼÊỀẾỆỂỄÌÍỊỈĨÒÓỌỎÕÔỒỐỘỔỖƠỜỚỢỞỠÙÚỤỦŨƯỪỨỰỬỮỲÝỴỶỸ]+$',
  'ku-IQ': '^[ئابپتجچحخدرڕزژسشعغفڤقکگلڵمنوۆھەیێيطؤثآإأكضصةظذ]+$',
  'ar': '^[ءآأؤإئابةتثجحخدذرزسشصضطظعغفقكلمنهوىيًٌٍَُِّْٰ]+',
  'he': '^[א-ת]+',
  'fa': '^[\'آاءأؤئبپتثجچحخدذرزژسشصضطظعغفقکگلمنوهةی\']+$',
  'hi-IN': '^[\u0900-\u0961]+[\u0972-\u097F]*$',
}

alphanumeric = {
  'en-US': '^[0-9A-Z]+$',
  'az-AZ': '^[0-9A-VXYZÇƏĞİıÖŞÜ]+$',
  'bg-BG': '^[0-9А-Я]+$',
  'cs-CZ': '^[0-9A-ZÁČĎÉĚÍŇÓŘŠŤÚŮÝŽ]+$',
  'da-DK': '^[0-9A-ZÆØÅ]+$',
  'de-DE': '^[0-9A-ZÄÖÜß]+$',
  'el-GR': '^[0-9Α-ω]+$',
  'es-ES': '^[0-9A-ZÁÉÍÑÓÚÜ]+$',
  'fr-FR': '^[0-9A-ZÀÂÆÇÉÈÊËÏÎÔŒÙÛÜŸ]+$',
  'it-IT': '^[0-9A-ZÀÉÈÌÎÓÒÙ]+$',
  'hu-HU': '^[0-9A-ZÁÉÍÓÖŐÚÜŰ]+$',
  'nb-NO': '^[0-9A-ZÆØÅ]+$',
  'nl-NL': '^[0-9A-ZÁÉËÏÓÖÜÚ]+$',
  'nn-NO': '^[0-9A-ZÆØÅ]+$',
  'pl-PL': '^[0-9A-ZĄĆĘŚŁŃÓŻŹ]+$',
  'pt-PT': '^[0-9A-ZÃÁÀÂÄÇÉÊËÍÏÕÓÔÖÚÜ]+$',
  'ru-RU': '^[0-9А-ЯЁ]+$',
  'sl-SI': '^[0-9A-ZČĆĐŠŽ]+$',
  'sk-SK': '^[0-9A-ZÁČĎÉÍŇÓŠŤÚÝŽĹŔĽÄÔ]+$',
  'sr-RS@latin': '^[0-9A-ZČĆŽŠĐ]+$',
  'sr-RS': '^[0-9А-ЯЂЈЉЊЋЏ]+$',
  'sv-SE': '^[0-9A-ZÅÄÖ]+$',
  'th-TH': '^[ก-๙\s]+$',
  'tr-TR': '^[0-9A-ZÇĞİıÖŞÜ]+$',
  'uk-UA': '^[0-9А-ЩЬЮЯЄIЇҐі]+$',
  'ku-IQ': '^[٠١٢٣٤٥٦٧٨٩0-9ئابپتجچحخدرڕزژسشعغفڤقکگلڵمنوۆھەیێيطؤثآإأكضصةظذ]+$',
  'vi-VN': '^[0-9A-ZÀÁẠẢÃÂẦẤẬẨẪĂẰẮẶẲẴĐÈÉẸẺẼÊỀẾỆỂỄÌÍỊỈĨÒÓỌỎÕÔỒỐỘỔỖƠỜỚỢỞỠÙÚỤỦŨƯỪỨỰỬỮỲÝỴỶỸ]+$',
  'ar': '^[٠١٢٣٤٥٦٧٨٩0-9ءآأؤإئابةتثجحخدذرزسشصضطظعغفقكلمنهوىيًٌٍَُِّْٰ]+',
  'he': '^[0-9א-ת]+',
  'fa': '^[\'0-9آاءأؤئبپتثجچحخدذرزژسشصضطظعغفقکگلمنوهةی۱۲۳۴۵۶۷۸۹۰\']+$',
  'hi-IN': '^[\u0900-\u0963]+[\u0966-\u097F]*$',
}

decimal = {
  'en-US': '.',
  'ar': '٫',
}


english_locales = ['AU', 'GB', 'HK', 'IN', 'NZ', 'ZA', 'ZM']

for i, locale in enumerate(english_locales):
    locale = "en-{}".format(english_locales[i])
    alpha[locale] = alpha['en-US']
    alphanumeric[locale] = alphanumeric['en-US']
    decimal[locale] = decimal['en-US']

arabic_locales = [
  'AE', 'BH', 'DZ', 'EG', 'IQ', 'JO', 'KW', 'LB', 'LY',
  'MA', 'QM', 'QA', 'SA', 'SD', 'SY', 'TN', 'YE',
]

for j, locale in enumerate(arabic_locales):
    locale = "ar-{}".format(arabic_locales[j])
    alpha[locale] = alpha['en-US']
    alphanumeric[locale] = alphanumeric['en-US']
    decimal[locale] = decimal['en-US']

farsi_locales = [
  'IR', 'AF',
]

for k, locale in enumerate(farsi_locales):
    locale = "fa-{}".format(farsi_locales[k])
    alpha[locale] = alpha['en-US']
    alphanumeric[locale] = alphanumeric['en-US']
    decimal[locale] = decimal['en-US']

dot_decimal = ['ar-EG', 'ar-LB', 'ar-LY']

for n, _ in enumerate(farsi_locales):
    decimal[dot_decimal[n]] = decimal['en-US']

comma_decimal = [
  'bg-BG', 'cs-CZ', 'da-DK', 'de-DE', 'el-GR', 'en-ZM', 'es-ES', 'fr-CA', 'fr-FR',
  'id-ID', 'it-IT', 'ku-IQ', 'hi-IN', 'hu-HU', 'nb-NO', 'nn-NO', 'nl-NL', 'pl-PL', 'pt-PT',
  'ru-RU', 'sl-SI', 'sr-RS@latin', 'sr-RS', 'sv-SE', 'tr-TR', 'uk-UA', 'vi-VN',
]

for m, _ in enumerate(comma_decimal):
    decimal[comma_decimal[m]] = decimal['en-US']

alpha['fr-CA'] = alpha['fr-FR']
alphanumeric['fr-CA'] = alphanumeric['fr-FR']

alpha['pt-BR'] = alpha['pt-PT']
alphanumeric['pt-BR'] = alphanumeric['pt-PT']
decimal['pt-BR'] = decimal['pt-PT']

alpha['pl-Pl'] = alpha['pl-PL']
alphanumeric['pl-Pl'] = alphanumeric['pl-PL']
decimal['pl-Pl'] = decimal['pl-PL']

alpha['fa-AF'] = alpha['fa']



import unittest
from pyvalidator import *

class TestIsAlpha(unittest.TestCase):

    def test_valid_alpha_strings(self):
        self.assertTrue(is_alpha('abc'))
        self.assertTrue(is_alpha('ABC'))
        self.assertTrue(is_alpha('FoObar'))
        print('OK - test_valid_alpha_strings')

    def test_invalid_alpha_strings(self):
        self.assertFalse(is_alpha('abc1'))
        self.assertFalse(is_alpha('  foo  '))
        self.assertFalse(is_alpha('FÜübar'))
        self.assertFalse(is_alpha('Jön'))
        print('OK - test_invalid_alpha_strings')

    def test_valid_alpha_strings_with_ignored_characters(self):
        self.assertTrue(is_alpha('en-US', 'en-US', { "ignore": '- /' }))
        self.assertTrue(is_alpha('this is a valid alpha string', 'en-US', { "ignore": '- /' }))
        self.assertTrue(is_alpha('us/usa', 'en-US', { "ignore": '- /' }))
        print('OK - test_valid_alpha_strings_with_ignored_characters')

    def test_invalid_alpha_strings_with_ignored_characters(self):
        self.assertFalse(is_alpha('1. this is not a valid alpha string', 'en-US', { "ignore": '- /' }))
        self.assertFalse(is_alpha('this$is also not a valid.alpha string', 'en-US', { "ignore": '- /' }))
        self.assertFalse(is_alpha('this is also not a valid alpha string.', 'en-US', { "ignore": '- /' }))
        print('OK - test_invalid_alpha_strings_with_ignored_characters')

    def test_should_throw_for_invalid_ignore_matcher(self):
        self.assertRaises(Exception, is_alpha, ['this is also not a valid alpha string.', 'en-US', { "ignore": 123 }])
        print('OK - test_should_throw_for_invalid_ignore_matcher')



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.merge import merge
from .utils.join import join

def currency_regex(options):
    decimal_digits = "\d{" + str(options["digits_after_decimal"][0]) + "}"
    for index, digit in enumerate(options["digits_after_decimal"]):
        if index != 0:
            decimal_digits = "{}|\d{}{}{}".format(decimal_digits, '{', digit, '}')
    
    def __match(x):
        return "\\{}".format(x.group(0))

    symbol = "({}){}".format(RegEx.sub("\W", __match, options["symbol"]), '' if options["require_symbol"] else '?')
    negative = '-?'
    whole_dollar_amount_without_sep = '[1-9]\d*'
    whole_dollar_amount_with_sep = "[1-9]\d{{0,2}}({}\d{{3}})*".format(options["thousands_separator"])
    valid_whole_dollar_amounts = [
        '0',
        whole_dollar_amount_without_sep,
        whole_dollar_amount_with_sep
    ]
    whole_dollar_amount = "({})?".format(join(valid_whole_dollar_amounts, '|'))
    decimal_amount = "(\{}({})){}".format(options["decimal_separator"], decimal_digits, '' if options["require_decimal"] else '?')
    pattern = "{}{}".format(
        whole_dollar_amount,
        decimal_amount if (options["allow_decimal"] or options["require_decimal"]) else ''
    )

    if options["allow_negatives"] and not options["parens_for_negatives"]:
        if options["negative_sign_after_digits"]:
            pattern += negative
        elif options["negative_sign_before_digits"]:
            pattern = negative + pattern

    if options["allow_negative_sign_placeholder"]:
        pattern = "( (?!\-))?" + pattern
    elif options["allow_space_after_symbol"]:
        pattern = " ?" + pattern
    elif options["allow_space_after_digits"]:
        pattern += '( (?!$))?'

    if options["symbol_after_digits"]:
        pattern += symbol
    else:
        pattern = symbol + pattern

    if options["allow_negatives"]:
        if options["parens_for_negatives"]:
            pattern = "(({})|{})".format(pattern, pattern)
        elif not (options["negative_sign_before_digits"] and options["negative_sign_after_digits"]):
            pattern = negative + pattern

    return RegEx("^(?!-? )(?=.*\d){}$".format(pattern))

default_currency_options = {
  "symbol": "$",
  "require_symbol": False,
  "allow_space_after_symbol": False,
  "symbol_after_digits": False,
  "allow_negatives": True,
  "parens_for_negatives": False,
  "negative_sign_before_digits": False,
  "negative_sign_after_digits": False,
  "allow_negative_sign_placeholder": False,
  "thousands_separator": ',',
  "decimal_separator": '.',
  "allow_decimal": True,
  "require_decimal": False,
  "digits_after_decimal": [2],
  "allow_space_after_digits": False,
}

def is_currency(input: str, options = {}) -> bool:
    assert_string(input)
    options = merge(options, default_currency_options)
    return bool(currency_regex(options).match(input))



import unittest
from pyvalidator import *

class TestIsDate(unittest.TestCase):

    def test_valid_date(self):
        self.assertTrue(is_date('2020/02/29'))
        self.assertTrue(is_date('2004-02-29'))
        self.assertTrue(is_date('2011-12-13'))
        self.assertTrue(is_date('1911-12-13'))
        print('OK - test_valid_date')

    def test_invalid_date(self):
        self.assertFalse(is_date('15072002'))
        self.assertFalse(is_date('2020-02-30'))
        self.assertFalse(is_date('2011-12/13'))
        self.assertFalse(is_date('2011/2/13'))
        self.assertFalse(is_date(''))
        self.assertFalse(is_date('asd'))
        print('OK - test_invalid_date')

    def test_valid_date_custom_format(self):
        self.assertTrue(is_date('12/20/19', { "format": "MM/DD/YY" }))
        self.assertTrue(is_date('12/20/19', { "format": "mm/dd/yy" }))
        self.assertTrue(is_date('02-04-04', { "format": "MM/DD/YY" }))
        self.assertTrue(is_date('02-04-04', { "format": "mm-DD-yy" }))
        print('OK - test_valid_date_custom_format')

    def test_invalid_date_custom_format(self):
        self.assertFalse(is_date('2/20/29', { "format": "MM/DD/YY" }))
        self.assertFalse(is_date('02/20/29', { "format": "MM/DD/YYYY" }))
        print('OK - test_invalid_date_custom_format')

    def test_valid_date_strict_mode(self):
        self.assertTrue(is_date('2020/02/29', { "strict_mode": True }))
        self.assertTrue(is_date('1920/02/29', { "strict_mode": True }))
        print('OK - test_valid_date_strict_mode')

    def test_invalid_date_strict_mode(self):
        self.assertFalse(is_date('15072002', { "strict_mode": True }))
        self.assertFalse(is_date('2020-02-30', { "strict_mode": True }))
        self.assertFalse(is_date('2004-02-29', { "strict_mode": True }))
        self.assertFalse(is_date('2011-12/13', { "strict_mode": True }))
        print('OK - test_invalid_date_strict_mode')

    def test_valid_date_strict_mode_hyphen_format(self):
        self.assertTrue(is_date('2020-02-29', { "strict_mode": True, "format": 'YYYY-MM-DD' }))
        print('OK - test_valid_date_strict_mode_hyphen_format')

    def test_invalid_date_strict_mode_hyphen_format(self):
        self.assertFalse(is_date('2020/02/29', { "strict_mode": True, "format": 'YYYY-MM-DD' }))
        self.assertFalse(is_date('2019/09/30', { "strict_mode": True, "format": 'YYYY-MM-DD' }))
        print('OK - test_invalid_date_strict_mode_hyphen_format')

    def test_valid_date_custom_delimiter(self):
        self.assertTrue(is_date('2020.02.29', { "delimiters": ['.'] }))
        self.assertTrue(is_date('2020,02,29', { "delimiters": [','] }))
        self.assertTrue(is_date('02,11,2019', { "delimiters": [','], "format": 'DD/MM/YYYY' }))
        self.assertTrue(is_date('02,11,19', { "delimiters": [','], "format": 'DD/MM/YY' }))
        self.assertTrue(is_date('02|11|19', { "delimiters": ['|'], "format": 'YY|MM|DD' }))
        self.assertTrue(is_date('02;11;19', { "delimiters": [';'], "format": 'YY;MM;DD' }))
        self.assertTrue(is_date('02:11:19', { "delimiters": [':'], "format": 'YY:MM:DD' }))
        print('OK - test_valid_date_custom_delimiter')

    def test_invalid_date_custom_delimiter(self):
        self.assertFalse(is_date('2020,02,29', { "delimiters": ['.'] }))
        self.assertFalse(is_date('2020.02.29', { "delimiters": [','] }))
        self.assertFalse(is_date('02,11,2019', { "delimiters": [','], "format": 'DD/MM/YY' }))
        print('OK - test_invalid_date_custom_delimiter')

    def test_valid_date_custom_delimiter_strict_mode(self):
        self.assertTrue(is_date('2020.02.29', { "delimiters": ['.'], "strict_mode": True, "format": 'YYYY.MM.DD' }))
        self.assertTrue(is_date('02.29.20', { "delimiters": ['.'], "strict_mode": True, "format": 'MM.DD.YY' }))
        print('OK - test_valid_date_custom_delimiter_strict_mode')

    def test_invalid_date_custom_delimiter_strict_mode(self):
        self.assertFalse(is_date('2020.02.29', { "delimiters": ['.'], "strict_mode": True, "format": 'YYYY,MM,DD' }))
        print('OK - test_invalid_date_custom_delimiter_strict_mode')

    def test_rais_error_for_unsuported_date_format(self):
        self.assertRaises(Exception, is_date, ['2020/02/29', { "format": 'MM/YY/DD' }])
        self.assertRaises(Exception, is_date, ['2020/02/29', { "format": '' }])
        print('OK - test_rais_error_for_unsuported_date_format')



import unittest
from pyvalidator import *
import datetime

class TestIsAfter(unittest.TestCase):

    def test_valid_after_dates_against_a_start_date(self):
        self.assertTrue(is_after('2011-08-04', '2011-08-03'))
        self.assertTrue(is_after(datetime.datetime.now().isoformat(), '2011-08-03'))
        print('OK - test_valid_after_dates_against_a_start_date')

    def test_invalid_after_dates_against_a_start_date(self):
        self.assertFalse(is_after('2011-07-02', '2011-08-03'))
        self.assertFalse(is_after('2011-08-03', '2011-08-03'))
        self.assertFalse(is_after(datetime.datetime.utcfromtimestamp(0).isoformat(), '2011-08-03'))
        self.assertFalse(is_after('foo', '2011-08-03'))
        print('OK - test_invalid_after_dates_against_a_start_date')



import unittest
from pyvalidator import *

class TestIsFloat(unittest.TestCase):

    def test_valid_floats(self):
        self.assertTrue(is_float('123'))
        self.assertTrue(is_float('123.'))
        self.assertTrue(is_float('123.123'))
        self.assertTrue(is_float('-123.123'))
        self.assertTrue(is_float('-0.123'))
        self.assertTrue(is_float('+0.123'))
        self.assertTrue(is_float('0.123'))
        self.assertTrue(is_float('.0'))
        self.assertTrue(is_float('-.123'))
        self.assertTrue(is_float('+.123'))
        self.assertTrue(is_float('01.123'))
        self.assertTrue(is_float('-0.11250738585072011e-307'))
        print('OK - test_valid_floats')

    def test_invalid_floats(self):
        self.assertFalse(is_float('+'))
        self.assertFalse(is_float('-'))
        self.assertFalse(is_float('  '))
        self.assertFalse(is_float(''))
        self.assertFalse(is_float('.'))
        self.assertFalse(is_float('foo'))
        self.assertFalse(is_float('20.foo'))
        self.assertFalse(is_float('2020-11-06T14:31:00.135Z'))
        print('OK - test_invalid_floats')

    def test_valid_floats_locale_ar(self):
        self.assertTrue(is_float('123', { "locale": "ar" }))
        self.assertTrue(is_float('123٫', { "locale": "ar" }))
        self.assertTrue(is_float('123٫123', { "locale": "ar" }))
        self.assertTrue(is_float('-123٫123', { "locale": "ar" }))
        self.assertTrue(is_float('-0٫123', { "locale": "ar" }))
        self.assertTrue(is_float('+0٫123', { "locale": "ar" }))
        self.assertTrue(is_float('0٫123', { "locale": "ar" }))
        self.assertTrue(is_float('٫0', { "locale": "ar" }))
        self.assertTrue(is_float('-٫123', { "locale": "ar" }))
        self.assertTrue(is_float('+٫123', { "locale": "ar" }))
        self.assertTrue(is_float('01٫123', { "locale": "ar" }))
        self.assertTrue(is_float('-0٫11250738585072011e-307', { "locale": "ar" }))
        print('OK - test_valid_floats_locale_ar')

    def test_invalid_floats_locale_ar(self):
        self.assertFalse(is_float('123,123', { "locale": "ar" }))
        self.assertFalse(is_float('123.123', { "locale": "ar" }))
        self.assertFalse(is_float('  ', { "locale": "ar" }))
        self.assertFalse(is_float('', { "locale": "ar" }))
        self.assertFalse(is_float('.', { "locale": "ar" }))
        self.assertFalse(is_float('foo', { "locale": "ar" }))
        self.assertFalse(is_float('20.foo', { "locale": "ar" }))
        self.assertFalse(is_float('2020-11-06T14:31:00.135Z', { "locale": "ar" }))
        print('OK - test_invalid_floats_locale_ar')



from .utils.assert_string import assert_string
from .is_base64 import is_base64

def is_jwt(input: str) -> bool:
    input = assert_string(input)

    dot_split = input.split('.')
    length = dot_split.length

    if length < 2 or length > 3:
        return False

    def reducer(acc: bool, curr: bool) -> bool:
        return acc and is_base64(curr, { "url_safe": True })

    return dot_split.reduce(reducer, True)



from .utils.assert_string import assert_string
from .utils.Classes.RegEx import RegEx

pattern = RegEx("^[\x00-\x7F]+$")

def is_ascii(input: str) -> bool:
    input = assert_string(input)
    return pattern.match(input)



from .utils.Classes.RegEx import RegEx
from .utils.assert_string import assert_string
from .utils.merge import merge
from .utils.includes import includesNot
from .alpha import decimal

def decimal_regexp(options):
    local = options["locale"]
    local_decimal = decimal[local]
    decimal_digits = options["decimal_digits"]
    require_decimal = '' if options["force_decimal"] else '?'
    regex = "^[-+]?([0-9]+)?(\{}[0-9]{{".format(local_decimal) + "{}}}){}$".format(decimal_digits, require_decimal)
    return RegEx(regex)

default_decimal_options = {
  "force_decimal": False,
  "decimal_digits": '1,',
  "locale": 'en-US',
}

blacklist = ['', '-', '+']

def is_decimal(input: str, options = {}) -> bool:
    input = assert_string(input)

    input = input.rstrip()

    options = merge(options, default_decimal_options)

    if options["locale"] not in decimal:
        raise Exception("Invalid locale '{}'".format(options["locale"]))

    without_white_space = input.sub(" ", '')
    return includesNot(blacklist, without_white_space) and bool(decimal_regexp(options).match(input))




from .utils.assert_string import assert_string

default_options = { "loose": False }
strict_booleans = ['true', 'false', '1', '0']
loose_booleans = [*strict_booleans, 'yes', 'no', 'y', 'n']

def is_boolean(input: str, options = default_options) -> bool:
    assert_string(input)

    input_lower_case = input.lower()

    if options["loose"]:
        return input_lower_case in loose_booleans

    return input_lower_case in strict_booleans




from typing import TypedDict, List as ListT, Tuple, Union, Literal

from .utils.Classes.RegEx import RegEx
from .utils.Classes.List import List
from .utils.Classes.String import String
from .utils.assert_string import assert_string
from .utils.merge import merge
from .utils.to_date import to_date

date_format_pattern = RegEx(r"(^(y{4}|y{2})[|:;,.\/-](m{1,2})[|:;,.\/-](d{1,2})$)|(^(m{1,2})[|:;,.\/-](d{1,2})[|:;,.\/-]((y{4}|y{2})$))|(^(d{1,2})[|:;,.\/-](m{1,2})[|:;,.\/-]((y{4}|y{2})$))", 'i')

class IsDateOptions(TypedDict):
    format: Union[
        Literal['DD/MM/YYYY'],
        Literal['D/MM/YYYY'],
        Literal['D/M/YYYY'],
        Literal['D/M/YY'],
        Literal['D/MM/YY'],
        Literal['DD/MM/YY'],
        Literal['YYYY/MM/DD'],
        Literal['YYYY/MM/D'],
        Literal['YYYY/M/D'],
        Literal['YY/M/D'],
        Literal['YY/MM/D'],
        Literal['YY/MM/DD'],
        Literal['MM/DD/YYYY'],
        Literal['M/DD/YYYY'],
        Literal['M/D/YYYY'],
        Literal['M/D/YY'],
        Literal['M/DD/YY'],
        Literal['MM/DD/YY'],
    ]
    delimiters: ListT[Union[
        Literal['|'],
        Literal[':'],
        Literal[';'],
        Literal[','],
        Literal['.'],
        Literal['-'],
        Literal['/'],
    ]]
    strict_mode: bool

__default_options: IsDateOptions = {
  "format": 'YYYY/MM/DD',
  "delimiters": ['/', '-'],
  "strict_mode": False,
}

def zip(date_list: ListT[str], format_list: ListT[str]) -> ListT[Tuple[str, str]]:
    zipped = []
    smaller_list = date_list if date_list.length < format_list.length else format_list
    for i, _ in enumerate(smaller_list):
        zipped.append([date_list[i], format_list[i]])
    return zipped

def is_date(input: str, options: IsDateOptions = {}) -> bool:
    input = assert_string(input)

    options = merge(options, __default_options)

    format = String(options['format'])

    if not date_format_pattern.match(format):
        raise Exception('Not supported format provided: ', format)

    format_delimiter: Union[str, None] = List(options['delimiters']).find(lambda delimiter, _: delimiter in format)

    date_delimiter = format_delimiter if options['strict_mode'] else '/'
    if not date_delimiter:
        return False

    input_delimiter: Union[str, None] = List(options['delimiters']).find(lambda delimiter, _: delimiter in input)

    if not input_delimiter:
        return False

    format = format if options['strict_mode'] else format.sub("\{}".format(input_delimiter), '/')
    input = input if options['strict_mode'] else input.sub("\{}".format(input_delimiter), '/')

    format_tupples = zip(
        input.split(date_delimiter),
        format.lower().split(date_delimiter)
    )

    date_dict = {}

    for format_tupple in format_tupples:
        if len(format_tupple[0]) != len(format_tupple[1]):
            return False
        format_char = format_tupple[1][0]
        date_dict[format_char] = format_tupple[0]

    return bool(to_date("{}/{}/{}".format(date_dict['m'], date_dict['d'], date_dict['y'])))



from .utils.assert_string import assert_string
from .is_hexadecimal import is_hexadecimal

def is_mongo_id(input: str) -> bool:
    input = assert_string(input)

    return input.length == 24 and is_hexadecimal(input)



import unittest
from pyvalidator import *

class TestIsHsl(unittest.TestCase):

    def test_valid_hsl(self):
        self.assertTrue(is_hsl('hsl(360,0000000000100%,000000100%)'))
        self.assertTrue(is_hsl('hsl(000010, 00000000001%, 00000040%)'))
        self.assertTrue(is_hsl('HSL(00000,0000000000100%,000000100%)'))
        self.assertTrue(is_hsl('hsL(0, 0%, 0%)'))
        self.assertTrue(is_hsl('hSl(  360  , 100%  , 100%   )'))
        self.assertTrue(is_hsl('Hsl(  00150  , 000099%  , 01%   )'))
        self.assertTrue(is_hsl('hsl(01080, 03%, 4%)'))
        self.assertTrue(is_hsl('hsl(-540, 03%, 4%)'))
        self.assertTrue(is_hsl('hsla(+540, 03%, 4%)'))
        self.assertTrue(is_hsl('hsla(+540, 03%, 4%, 500)'))
        self.assertTrue(is_hsl('hsl(+540deg, 03%, 4%, 500)'))
        self.assertTrue(is_hsl('hsl(+540gRaD, 03%, 4%, 500)'))
        self.assertTrue(is_hsl('hsl(+540.01e-98rad, 03%, 4%, 500)'))
        self.assertTrue(is_hsl('hsl(-540.5turn, 03%, 4%, 500)'))
        self.assertTrue(is_hsl('hsl(+540, 03%, 4%, 500e-01)'))
        self.assertTrue(is_hsl('hsl(+540, 03%, 4%, 500e+80)'))
        self.assertTrue(is_hsl('hsl(4.71239rad, 60%, 70%)'))
        self.assertTrue(is_hsl('hsl(270deg, 60%, 70%)'))
        self.assertTrue(is_hsl('hsl(200, +.1%, 62%, 1)'))
        self.assertTrue(is_hsl('hsl(270 60% 70%)'))
        self.assertTrue(is_hsl('hsl(200, +.1e-9%, 62e10%, 1)'))
        self.assertTrue(is_hsl('hsl(.75turn, 60%, 70%)'))
        self.assertTrue(is_hsl('hsl(200grad +.1% 62% / 1)'))
        self.assertTrue(is_hsl('hsl(270, 60%, 50%, .15)'))
        self.assertTrue(is_hsl('hsl(270, 60%, 50%, 15%)'))
        self.assertTrue(is_hsl('hsl(270 60% 50% / .15)'))
        self.assertTrue(is_hsl('hsl(270 60% 50% / 15%)'))
        print('OK - test_valid_hsl')

    def test_invalid_hsl(self):
        self.assertFalse(is_hsl('hsl (360,0000000000100%,000000100%)'))
        self.assertFalse(is_hsl('hsl(0260, 100 %, 100%)'))
        self.assertFalse(is_hsl('hsl(0160, 100%, 100%, 100 %)'))
        self.assertFalse(is_hsl('hsl(-0160, 100%, 100a)'))
        self.assertFalse(is_hsl('hsl(-0160, 100%, 100)'))
        self.assertFalse(is_hsl('hsl(-0160 100%, 100%, )'))
        self.assertFalse(is_hsl('hsl(270 deg, 60%, 70%)'))
        self.assertFalse(is_hsl('hsl( deg, 60%, 70%)'))
        self.assertFalse(is_hsl('hsl(, 60%, 70%)'))
        self.assertFalse(is_hsl('hsl(3000deg, 70%)'))
        print('OK - test_invalid_hsl')



import unittest
from pyvalidator import *

class TestIsMd5(unittest.TestCase):

    def test_valid_md5(self):
        self.assertTrue(is_md5('d94f3f016ae679c3008de268209132f0'))
        self.assertTrue(is_md5('751adbc511ccbe8edf23d486fa4581c0'))
        self.assertTrue(is_md5('88dae00e614d8f24cfd5a8b3f8002e99'))
        self.assertTrue(is_md5('0bf1c35032a71a14c2f719e5a14c1e99'))
        print('OK - test_valid_md5')

    def test_invalid_md5(self):
        self.assertFalse(is_md5('KYT0bf1c35032a71a14c2f719e5a14c1'))
        self.assertFalse(is_md5('q94375dj93458w31'))
        self.assertFalse(is_md5('39485729341'))
        self.assertFalse(is_md5('%&FHKJFvc'))
        self.assertFalse(is_md5(''))
        print('OK - test_invalid_md5')

    def test_fail_md5(self):
        self.assertRaises(Exception, is_md5, None)
        self.assertRaises(Exception, is_md5, {})



import unittest
from pyvalidator import *

class TestIsLowercase(unittest.TestCase):

    def test_valid_lowercase(self):
        self.assertTrue(is_lowercase('ccc'))
        self.assertTrue(is_lowercase('qwerty'))
        self.assertTrue(is_lowercase('asd123'))
        self.assertTrue(is_lowercase('hello world'))
        self.assertTrue(is_lowercase('   .'))
        print('OK - test_valid_lowercase')

    def test_invalid_lowercase(self):
        self.assertFalse(is_lowercase('hello WoRLD'))
        self.assertFalse(is_lowercase('HELLO WORLd'))
        self.assertFalse(is_lowercase('ASD123'))
        print('OK - test_invalid_lowercase')



from typing import Union

from .utils.assert_string import assert_string
from .utils.to_int import to_int
from .utils.to_float import to_float

def is_divisible_by(input: str, num: Union[int, float, str]) -> bool:
    assert_string(input)
    if to_float(input) != None and to_int(num) != None:
        return (to_float(input) % to_int(num)) == 0
    return False



import unittest
from pyvalidator import *

class TestIsUuid(unittest.TestCase):

    def test_valid_uuid(self):
        self.assertTrue(is_uuid('A987FBC9-4BED-3078-CF07-9141BA07C9F3'))
        self.assertTrue(is_uuid('A987FBC9-4BED-4078-8F07-9141BA07C9F3'))
        self.assertTrue(is_uuid('A987FBC9-4BED-5078-AF07-9141BA07C9F3'))
        print('OK - test_valid_uuid')

    def test_invalid_uuid(self):
        self.assertFalse(is_uuid(''))
        self.assertFalse(is_uuid('xxxA987FBC9-4BED-3078-CF07-9141BA07C9F3'))
        self.assertFalse(is_uuid('A987FBC9-4BED-3078-CF07-9141BA07C9F3xxx'))
        self.assertFalse(is_uuid('A987FBC94BED3078CF079141BA07C9F3'))
        self.assertFalse(is_uuid('114812'))
        self.assertFalse(is_uuid(','))
        self.assertFalse(is_uuid('foofoo'))
        self.assertFalse(is_uuid('987FBC9-4BED-3078-CF07A-9141BA07C911'))
        self.assertFalse(is_uuid('AAAAAAAA-1111-2222-AAAG-111111111111'))
        print('OK - test_invalid_uuid')

    def test_valid_uuid_version_1(self):
        self.assertTrue(is_uuid('E034B584-7D89-11E9-9669-1AECF481A97B', "1"))
        self.assertTrue(is_uuid('E034B584-7D89-11E9-9669-1AECF481A97B', 1))
        print('OK - test_valid_uuid_version_1')

    def test_invalid_uuid_version_1(self):
        self.assertFalse(is_uuid('', "1"))
        self.assertFalse(is_uuid('xxxA987FBC9-4BED-3078-CF07-9141BA07C9F3', "1"))
        self.assertFalse(is_uuid('AAAAAAAA-1111-2222-AAAG', "1"))
        self.assertFalse(is_uuid('AAAAAAAA-1111-2222-AAAG-111111111111', "1"))
        self.assertFalse(is_uuid('A987FBC9-4BED-4078-8F07-9141BA07C9F3', "1"))
        self.assertFalse(is_uuid('A987FBC9-4BED-5078-AF07-9141BA07C911', "1"))
        self.assertFalse(is_uuid('A987FBC9-4BED-5078-AF07-9141BA07C911', 1))
        print('OK - test_invalid_uuid_version_1')

    def test_valid_uuid_version_2(self):
        self.assertTrue(is_uuid('A117FBC9-4BED-2078-CF07-9141BA07C9F3', "2"))
        self.assertTrue(is_uuid('A117FBC9-4BED-2078-CF07-9141BA07C9F3', 2))
        print('OK - test_valid_uuid_version_2')

    def test_invalid_uuid_version_2(self):
        self.assertFalse(is_uuid('', "2"))
        self.assertFalse(is_uuid('xxxA987FBC9-4BED-3078-CF07-9141BA07C9F3', "2"))
        self.assertFalse(is_uuid('AAAAAAAA-1111-1111-AAAG', "2"))
        self.assertFalse(is_uuid('AAAAAAAA-1111-1111-AAAG-111111111112', "2"))
        self.assertFalse(is_uuid('934859', "2"))
        self.assertFalse(is_uuid('A987FBC9-4BED-5078-AF07-9141BA07C911', "2"))
        self.assertFalse(is_uuid('A987FBC9-4BED-5078-AF07-9141BA07C9F2', "2"))
        self.assertFalse(is_uuid('A987FBC9-4BED-5078-AF07-9141BA07C9F2', 2))
        print('OK - test_invalid_uuid_version_2')

    def test_valid_uuid_version_3(self):
        self.assertTrue(is_uuid('1987FBC9-4BED-3078-CF07-9141BA07C9F3', "3"))
        self.assertTrue(is_uuid('1987FBC9-4BED-3078-CF07-9141BA07C9F3', 3))
        print('OK - test_valid_uuid_version_3')

    def test_invalid_uuid_version_3(self):
        self.assertFalse(is_uuid('', "3"))
        self.assertFalse(is_uuid('xxxA987FBC9-4BED-3078-CF07-9141BA07C9F3', "3"))
        self.assertFalse(is_uuid('AAAAAAAA-1111-1111-AAAG-111111111112', "3"))
        self.assertFalse(is_uuid('934859', "3"))
        self.assertFalse(is_uuid('A987FBC9-4BED-4078-8F07-9141BA07C9F3', "3"))
        self.assertFalse(is_uuid('A987FBC9-4BED-5078-8F07-9141BA07C9F3', "3"))
        print('OK - test_invalid_uuid_version_3')

    def test_valid_uuid_version_4(self):
        self.assertTrue(is_uuid('713ae7e3-cb32-45f9-adcb-7c4fa86b90c1', "4"))
        self.assertTrue(is_uuid('625e63f3-58f5-40b7-83a1-a72ad31acffb', "4"))
        self.assertTrue(is_uuid('57b73598-8764-4ad0-a76a-679bb6640eb1', "4"))
        self.assertTrue(is_uuid('9c858901-8a57-4791-81fe-4c455b099bc9', "4"))
        print('OK - test_valid_uuid_version_4')

    def test_invalid_uuid_version_4(self):
        self.assertFalse(is_uuid('', "4"))
        self.assertFalse(is_uuid('xxxA987FBC9-4BED-3078-CF07-9141BA07C9F3', "4"))
        self.assertFalse(is_uuid('AAAAAAAA-1111-1111-AAAG-111111111112', "4"))
        self.assertFalse(is_uuid('934859', "4"))
        self.assertFalse(is_uuid('A987FBC9-4BED-3078-8F07-9141BA07C9F3', "4"))
        self.assertFalse(is_uuid('A987FBC9-4BED-5078-8F07-9141BA07C9F3', "4"))
        print('OK - test_invalid_uuid_version_4')

    def test_valid_uuid_version_5(self):
        self.assertTrue(is_uuid('987FBC97-4BED-5078-AF07-9141BA07C9F3', "5"))
        self.assertTrue(is_uuid('987FBC97-4BED-5078-BF07-9141BA07C9F3', "5"))
        self.assertTrue(is_uuid('987FBC97-4BED-5078-8F07-9141BA07C9F3', "5"))
        self.assertTrue(is_uuid('987FBC97-4BED-5078-9F07-9141BA07C9F3', "5"))
        self.assertTrue(is_uuid('987FBC97-4BED-5078-9F07-9141BA07C9F3', 5))
        print('OK - test_valid_uuid_version_5')

    def test_invalid_uuid_version_5(self):
        self.assertFalse(is_uuid('', "5"))
        self.assertFalse(is_uuid('xxxA987FBC9-4BED-3078-CF07-9141BA07C9F3', "5"))
        self.assertFalse(is_uuid('934859', "5"))
        self.assertFalse(is_uuid('AAAAAAAA-1111-1111-AAAG-111111111111', "5"))
        self.assertFalse(is_uuid('9c858901-8a57-4791-81fe-4c455b099bc9', "5"))
        self.assertFalse(is_uuid('A987FBC9-4BED-3078-CF07-9141BA07C9F3', "5"))
        print('OK - test_invalid_uuid_version_5')

    def test_invalid_uuid_version_6(self):
        self.assertFalse(is_uuid('987FBC97-4BED-1078-AF07-9141BA07C9F3', "6"))
        self.assertFalse(is_uuid('987FBC97-4BED-2078-AF07-9141BA07C9F3', "6"))
        self.assertFalse(is_uuid('987FBC97-4BED-3078-AF07-9141BA07C9F3', "6"))
        self.assertFalse(is_uuid('987FBC97-4BED-4078-AF07-9141BA07C9F3', "6"))
        self.assertFalse(is_uuid('987FBC97-4BED-5078-AF07-9141BA07C9F3', "6"))
        print('OK - test_invalid_uuid_version_6')



import unittest
from pyvalidator import *

class TestIsBtcAddress(unittest.TestCase):

    def test_valid_btc_address(self):
        self.assertTrue(is_btc_address('1MUz4VMYui5qY1mxUiG8BQ1Luv6tqkvaiL'))
        self.assertTrue(is_btc_address('3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy'))
        self.assertTrue(is_btc_address('bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq'))
        self.assertTrue(is_btc_address('14qViLJfdGaP4EeHnDyJbEGQysnCpwk3gd'))
        self.assertTrue(is_btc_address('35bSzXvRKLpHsHMrzb82f617cV4Srnt7hS'))
        self.assertTrue(is_btc_address('17VZNX1SN5NtKa8UQFxwQbFeFc3iqRYhemt'))
        self.assertTrue(is_btc_address('bc1qw508d6qejxtdg4y5r3zarvary0c5xw7kv8f3t4'))
        print('OK - test_valid_btc_address')

    def test_invalid_btc_address(self):
        self.assertFalse(is_btc_address('4J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy'))
        self.assertFalse(is_btc_address('0x56F0B8A998425c53c75C4A303D4eF987533c5597'))
        self.assertFalse(is_btc_address('pp8skudq3x5hzw8ew7vzsw8tn4k8wxsqsv0lt0mf3g'))
        self.assertFalse(is_btc_address('17VZNX1SN5NlKa8UQFxwQbFeFc3iqRYhem'))
        self.assertFalse(is_btc_address('BC1QW508D6QEJXTDG4Y5R3ZARVAYR0C5XW7KV8F3T4'))
        print('OK - test_invalid_btc_address')
